<?php
/**
 * @package    RSForm! Pro
 * @copyright  (c) 2013 - 2021 RSJoomla!
 * @link       https://www.rsjoomla.com
 * @license    GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Class RSFormProTargetPay
 */
class RSDirectoryTargetPay
{
	/**
	 * @var array
	 */
	public $banks;

	/**
	 * @var Sisow
	 */
	public $sale;

	/**
	 * @var array
	 */
	public $args = array();

	/**
	 * @return RSDirectoryTargetPay
	 */
	public static function getInstance($params = null)
	{
		static $inst;
		if (!$inst)
		{
			$inst = new RSDirectoryTargetPay($params);
		}

		return $inst;
	}

	/**
	 * RSDirectoryTargetPay constructor.
	 */
	public function __construct($params)
	{
		// just for
		if (is_null($params))
		{
			return;
		}

		$sale = new RSDirectoryTargetPayHelper($params->get('targetpaylayoutcode', ''), $params->get('mode', 1) ? false : true);

		$issuers = array();
		$banks   = array();

		$sale->DirectoryRequest($issuers, false);

		foreach ($issuers as $issuer => $value)
		{
			$banks[] = array('id' => $issuer, 'name' => $value);
		}

		$this->banks = $banks;
		$this->sale  = $sale;
	}
}

class RSDirectoryTargetPayHelper
{
	private $layoutcode;
	private $test;

	public $trxid;
	public $url;
	public $status;

	public function __construct($layoutcode, $test = false)
	{
		if (empty($layoutcode))
		{
			throw new Exception('No layout code provided!');
		}
		$this->layoutcode = $layoutcode;

		$this->test = $test;
	}


	// DirectoryRequest
	public function DirectoryRequest(&$output, $select = false)
	{
		if ($select === true)
		{
			$output[] = "<select id=\"sisowbank\" name=\"issuerid\">";
		}

		$items = array();
		$url   = 'https://www.targetpay.com/ideal/getissuers.php?ver=3&format=xml';

		$http = JHttpFactory::getHttp();

		if ($response = $http->get($url, array(), 5))
		{
			if ($response->code != 200)
			{
				throw new Exception(JText::sprintf('PLG_SYSTEM_RSDIRECTORYIDEAL_CONNECTION_ERROR', $response->code));
			}

			$issuers = simplexml_load_string($response->body);
			foreach ($issuers->issuer as $issuer)
			{
				$items[] = array(
					'id'   => (string) $issuer['id'],
					'name' => (string) $issuer
				);
			}
		}

		if ($items)
		{
			foreach ($items as $item)
			{
				if ($select === true)
				{
					$output [] = "<option value=\"" . $item['id'] . "\">" . $item['name'] . "</option>";
				}
				else
				{
					$output[$item['id']] = $item['name'];
				}
			}
		}

		if ($select === true)
		{
			$output[] = "</select>";

			$output = implode('', $output);
		}

		return;
	}

	public function TransactionRequest($args = null)
	{
		if (is_null($args))
		{
			throw new Exception('No arguments sent!');
		}

		if (!isset($args['rtlo']) || strlen($args['rtlo']) === 0)
		{
			$args['rtlo'] = $this->layoutcode;
		}

		if ($this->test)
		{
			$args['test'] = 1;
		}


		$url  = 'https://www.targetpay.com/ideal/start?' . http_build_query($args);
		$http = JHttpFactory::getHttp();

		if ($response = $http->get($url, null, 5))
		{
			if ($response->code != 200)
			{
				throw new Exception(JText::sprintf('PLG_SYSTEM_RSDIRECTORYIDEAL_CONNECTION_ERROR', $response->code));
			}

			$parts   = explode(' ', $response->body, 2);
			$type    = $parts[0];
			$message = isset($parts[1]) ? $parts[1] : '';

			if ($type == '000000' && $message && strpos($message, '|') !== false)
			{
				list($trxid, $url) = explode('|', $message);

				$this->trxid = $trxid;
				$this->url = $url;
			}
			elseif ($message)
			{
				throw new Exception($message);
			}
		}
	}


	public function StatusRequest($args = null)
	{
		if (is_null($args))
		{
			throw new Exception('No arguments sent!');
		}

		if (!isset($args['trxid']) || strlen($args['trxid']) === 0)
		{
			throw new Exception('No transaction ID');
		}

		if (!isset($args['rtlo']) || strlen($args['rtlo']) === 0)
		{
			$args['rtlo'] = $this->layoutcode;
		}

		if ($this->test)
		{
			$args['test'] = 1;
		}

		$url  = 'https://www.targetpay.com/ideal/check?' . http_build_query($args);
		$http = JHttpFactory::getHttp();

		if ($response = $http->get($url, null, 5))
		{
			if ($response->code != 200)
			{
				throw new Exception(JText::sprintf('PLG_SYSTEM_RSDIRECTORYIDEAL_CONNECTION_ERROR', $response->code));
			}

			if ($response->body == '000000 OK')
			{
				$this->status = 'Success';
			}
			else
			{
				$this->status = 'Error';
			}
		}
	}
}