<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('list');

/**
 * Category Select options field.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldMosetscategoryselect extends JFormFieldList
{
    /**
     * A flexible category list that respects access controls
     *
     * @var string
     */
    public $type = 'mosetscategoryselect';

    /**
     * Method to get a list of categories that respects access controls and can be used for
     * either category assignment or parent category assignment in edit screens.
     * Use the parent element to indicate that the field will be used for assigning parent categories.
     *
     * @return array The field option objects.
     */
    protected function getOptions()
    {
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
            ->select( $db->qn( array('cat_id', 'cat_parent', 'cat_name') ) )
            ->from( $db->qn('#__mt_cats', 'o') )
            ->order( $db->qn('ordering') );

        $db->setQuery($query);
        $items = $db->loadObjectList();


        $tree = RSDirectoryHelper::arrayToTree($items, 0, 'cat_id', 'cat_parent');
        $options = $this->mosetsTreeToOptions($tree);

        return $options;
    }

    /**
     * Method to generate an options array from a SobiPro categories tree.
     *
     * @access public
     *
     * @static
     *
     * @param array $tree
     * @param int $level
     *
     * @return array
     */
    protected function mosetsTreeToOptions($tree, $level = 0)
    {
        $options = array();

        if ($level == 0)
        {
            $options[] = JHtml::_('select.option', '', JText::_('PLG_RSDIRECTORY_IMPORTMOSETSTREE_CATEGORY_OPTION'), 'value', 'text');
        }

        if ($tree)
        {
            foreach ($tree as $item)
            {
                $text = str_repeat('- ', $level) . $item->cat_name;
                $disable = !empty($item->children);

                $options[] = JHtml::_('select.option', $item->cat_id, $text, 'value', 'text', $disable);

                if ( !empty($item->children) )
                {
                    if ( $options_aux = $this->mosetsTreeToOptions($item->children, $level + 1) )
                    {
                        $options = array_merge($options, $options_aux);
                    }
                }
            }
        }

        return $options;
    }
}