<?php
/**
 * @package	RSDirectory!
 * @copyright	(c) 2013 - 2021 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('list');

/**
 * Category Select options field.
 *
 * @package RSDirectory!
 * @subpackage com_rsdirectory
 */
class JFormFieldSobicategoryselect extends JFormFieldList
{
    /**
     * A flexible category list that respects access controls
     *
     * @var string
     */
    public $type = 'sobicategoryselect';

    /**
     * Method to get a list of categories that respects access controls and can be used for
     * either category assignment or parent category assignment in edit screens.
     * Use the parent element to indicate that the field will be used for assigning parent categories.
     *
     * @return array The field option objects.
     */
    protected function getOptions()
    {
        $db = JFactory::getDbo();
        $query = $db->getQuery(true)
            ->select( $db->qn( array('o.id', 'o.parent', 'o.name', 'o.oType') ) )
            ->from( $db->qn('#__sobipro_object', 'o') )
            ->leftJoin( $db->qn('#__sobipro_category', 'c') . ' ON ' . $db->qn('o.id') . ' = ' . $db->qn('c.id') )
            ->where( $db->qn('o.oType') . ' = ' . $db->q('section') . ' OR ' . $db->qn('o.oType') . ' = ' . $db->q('category') )
            ->order( $db->qn('c.position') );

        $db->setQuery($query);
        $items = $db->loadObjectList();


        $tree = RSDirectoryHelper::arrayToTree($items, 0, 'id', 'parent');
        $options = $this->sobiProTreeToOptions($tree);

        return $options;
    }

    /**
     * Method to generate an options array from a SobiPro categories tree.
     *
     * @access public
     *
     * @static
     *
     * @param array $tree
     * @param int $level
     *
     * @return array
     */
    protected function sobiProTreeToOptions($tree, $level = 0)
    {
        $options = array();

        if ($level == 0)
        {
            $options[] = JHtml::_('select.option', '', JText::_('PLG_RSDIRECTORY_IMPORTSOBIPRO_CATEGORY_OPTION'), 'value', 'text');
        }

        if ($tree)
        {
            foreach ($tree as $item)
            {
                $text = str_repeat('- ', $level) . $item->name;
                $disable = !empty($item->children) || $item->oType == 'section';

                $options[] = JHtml::_('select.option', $item->id, $text, 'value', 'text', $disable);

                if ( !empty($item->children) )
                {
                    if ( $options_aux = $this->sobiProTreeToOptions($item->children, $level + 1) )
                    {
                        $options = array_merge($options, $options_aux);
                    }
                }
            }
        }

        return $options;
    }
}