<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! More From This Module Helper.
 */
abstract class RSDirectoryMoreFromThisHelper {
	
	public static $params;
	
    /**
     * Get the related entries.
     *
     * @access public
     *
     * @static
     *
     * @return mixed
     */
    public static function getEntries() {
		// Get the JInput object.
		$jinput = JFactory::getApplication()->input;
		
		// Get entry id.    
		$entry_id = $jinput->getInt('id');
		
		// Get entry.
		$entry = RSDirectoryHelper::getEntry($entry_id);
		
		// Get DBO.
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true)
				->select($db->qn('e') . '.*')
				->from($db->qn('#__rsdirectory_entries', 'e'))
				->innerJoin($db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id'))
				->innerJoin($db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id'))
				->where($db->qn('e.id') . ' != ' . $db->q($entry_id))
				->where($db->qn('e.user_id') . ' = ' . $db->q($entry->user_id))
				->where($db->qn('e.published') . ' = ' . $db->q(1))
				->where($db->qn('u.block') . ' = ' . $db->q(0))
				->where('(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' > ' . $db->q(JFactory::getDate()->toSql()) . ')');

		if (self::$params->get('from_same_category')) {
			$query->where($db->qn('e.category_id') . ' = ' . $db->q($entry->category_id));
		} else {
			// Must only view entries from categories we have access to
			$categories_ids = self::getCategories();
			
			if (empty($categories_ids)) {
				$categories_ids = array(0);
			}

			if ($categories_ids) {
				// Add the categories condition.
				$query->where($db->qn('e.category_id') . ' IN (' . implode(',', array_unique($categories_ids)) . ')');
			}
		}
			   
		if (self::$params->get('enable_price_variation')) {
			$percentage = $entry->price * ((int)self::$params->get('price_variation') / 100);
			$query->where($db->qn('e.price') . ' >= ' . $db->q($entry->price - $percentage))
			      ->where($db->qn('e.price') . ' <= ' . $db->q($entry->price + $percentage));
		}
		
		$query->order($db->qn('e.promoted') . ' DESC');
		$max_entries = self::$params->get('max_entries', 3);
		
		// Ensure that max_entries is a factor of 12.
		if (12 % $max_entries != 0) {
			$max_entries = 3;	
		}
		
		$db->setQuery($query, 0, $max_entries);
		$entries = $db->loadObjectList();
			
		if (self::$params->get('display_thumbs')) {
			return RSDirectoryHelper::getEntriesData($entries);
		}
		
		return $entries; 
    }
	
	protected static function getCategories() {
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);
			
		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get('root')) {
			if ($children = $root->getChildren()) {				
				return self::getIds($children);
			}
		}
		
		return false;
	}
	
	protected static function getIds($categories) {
		$ids = array();

		foreach ($categories as $category) {
			$ids[] = $category->id;
			
			if ($category->hasChildren()) {
				$ids = array_merge($ids, self::getIds($category->getChildren()));
			}
		}
		
		return $ids;
	}
}