<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Popular Entries Module Helper.
 */
abstract class RSDirectoryPopularEntriesHelper {
	
	public static $params;
	
	protected static $featured_categories;
	
    /**
     * Get the most popular entries.
     *
     * @access public
     *
     * @static
     *
     * @return mixed
     */
    public static function getEntries() {
		// Get DBO.
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true)
				->select($db->qn('e') . '.*')
				->from($db->qn('#__rsdirectory_entries', 'e'))
				->innerJoin($db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id'))
				->innerJoin($db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id'))
				->where($db->qn('e.published') . ' = ' . $db->q(1))
				->where($db->qn('u.block') . ' = ' . $db->q(0))
				->where('(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' > ' . $db->q( JFactory::getDate()->toSql()) . ')')
				->order($db->qn('e.hits') . ' DESC, ' . $db->qn('e.promoted') . ' DESC');
				
		$categories_ids = self::getCategories();
		
		if (empty($categories_ids)) {
			$categories_ids = array(0);
		}
		
		if ($categories_ids) {
			// Add the categories condition.
			$query->where($db->qn('e.category_id') . ' IN (' . implode(',', array_unique($categories_ids)) . ')');
		}
		
		$max_entries = self::$params->get('max_entries', 3);
			
		// Ensure that max_entries is a factor of 12.
		if (12 % $max_entries != 0) {
			$max_entries = 3;
		}
		
		$db->setQuery($query, 0, $max_entries);
		$entries = $db->loadObjectList();
		
		if (self::$params->get('display_thumbs')) {
			return RSDirectoryHelper::getEntriesData($entries);
		}
		
		return $entries; 
    }
	
	protected static function getCategories() {
		self::$featured_categories = RSDirectoryHelper::arrayInt(self::$params->get('featured_categories'));
		if (empty(self::$featured_categories) || in_array(0, self::$featured_categories)) {
			self::$featured_categories = array();
		}
		
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);
			
		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get('root')) {
			if ($children = $root->getChildren()) {				
				return self::getIds($children);
			}
		}
		
		return false;
	}
	
	protected static function getIds($categories) {
		$ids = array();

		foreach ($categories as $category) {
			$has_children = $category->hasChildren();
			$children = null;

			// if features categories are defined search inside
			if (self::$featured_categories) {
				$children_ids = array();

				if ($has_children) {
					$children = $category->getChildren();

					foreach ($children as $child) {
						$children_ids[] = $child->id;
					}

					$intersect = array_intersect(self::$featured_categories, $children_ids);

					// if the child categories are not in the featured categories but the parent is add all the children to the ids
					if (count($intersect) == 0 && in_array($category->id, self::$featured_categories)) {
						$tmp = array_merge(self::$featured_categories, $children_ids);
						self::$featured_categories = array_unique($tmp);
					}
				}

				// if the category is in the featured categories add it to the ids
				if (in_array($category->id, self::$featured_categories)) {
					$ids[] = $category->id;
				}
			} else {
				// if no featured categories put all the ids
				$ids[] = $category->id;
			}

			// if the category has any children parse them too
			if ($has_children) {
				$ids = array_merge($ids, self::getIds((!is_null($children) ? $children : $category->getChildren())));
			}
		}

		return $ids;
	}
}