<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Recently Visited Entries Module Helper.
 */
abstract class RSDirectoryPromotedEntriesHelper {
    /**
     * Get the top rated entries.
     *
     * @access public
     *
     * @static
     *
     * @param JRegistry $params
     *
     * @return mixed
     */
    public static function getEntries($params) {
		// establish the ordering
		$order_type = $params->get('display_entries_type', 1);

		$order = '';
		switch($order_type) {
			case 1:
				$order = 'RAND()';
			break;

			case 2:
				$order = 'e.published_time DESC';
			break;

			case 3:
				$order = 'e.hits DESC';
			break;
		}
		// Get DBO.
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true)
				->select($db->qn('e') . '.*')
				->from($db->qn('#__rsdirectory_entries', 'e'))
				->innerJoin($db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id'))
				->innerJoin($db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id'))
				->where($db->qn('e.published') . ' = ' . $db->q(1))
				->where($db->qn('e.promoted') . ' = ' . $db->q(1))
				->where($db->qn('u.block') . ' = ' . $db->q(0))
				->where('(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' > ' . $db->q(JFactory::getDate()->toSql()) . ')')
				->order($order);

		$categories_ids = array();

		// get listed category or the entry category
		$jinput	= JFactory::getApplication()->input;
		$view = $jinput->get('view');
		$entry_id = $view == 'entry' ? (int) $jinput->get('id', 0) : 0;

		if ($view == 'entries' && $params->get('category_listing', 0)) {
			// Get categories filter.
			list($categories) = $jinput->get('categories', array(''), 'array');

			if (trim($categories) !== '') {
				$categories_ids = RSDirectoryHelper::arrayInt(explode(',', $categories));
			}
			
			// if a category is selected by the listing add it to the selected ones
			$selected_category = (int) $jinput->get('category', '');
			if (!empty($selected_category)) {
				$categories_ids[] = $selected_category;
			}

			$menuitem_params = JFactory::getApplication()->getParams();

			if (!$categories_ids && $menuitem_params->get('featured_categories')) {
				foreach ($menuitem_params->get('featured_categories') as $category) {
					// Reset the featured categories array if the category is 0 or '' ("All categories" is selected) as this will make the getFilterFields to use all the available categories.
					if (!$category) {
						$categories_ids = array();
						break;
					}
					$categories_ids[] = $category;
				}
			}

			if (!empty($categories_ids)){
				// let's get event the childes of the selected categories
				foreach ($categories_ids as $cat_id){
					if ($category_tree = self::getCategories($cat_id)) {
						$categories_ids = array_merge($categories_ids, $category_tree);
					}
				}
			} else {
				$categories_ids = self::getCategories();
			}

		} else if($view == 'entry' && $entry_id) {
			if ($entry = RSDirectoryHelper::getEntry($entry_id)) {
				$categories_ids[] = $entry->category_id;
			}
		} else {
			$categories_ids = self::getCategories();
		}

		if (empty($categories_ids)) {
			$categories_ids = array(0);
		}
		
		if ($categories_ids) {
			// Add the categories condition.
			$query->where($db->qn('e.category_id') . ' IN (' . implode(',', array_unique($categories_ids)) . ')');
		}
				
		$max_entries = $params->get('max_entries', 3);
			
		// Ensure that max_entries is a factor of 12.
		if (12 % $max_entries != 0) {
			$max_entries = 3;	
		}
		
		$db->setQuery($query, 0, $max_entries);
		$entries = $db->loadObjectList();
		
		if ($params->get('display_thumbs')) {
			return RSDirectoryHelper::getEntriesData($entries);
		}
		
		return $entries; 
    }
	
	protected static function getCategories($cat_id = 'root') {
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);
			
		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get($cat_id)) {
			if ($children = $root->getChildren()) {
				return self::getIds($children);
			}
		}
		
		return false;
	}
	
	protected static function getIds($categories) {
		$ids = array();

		foreach ($categories as $category) {
			$ids[] = $category->id;
			
			if ($category->hasChildren()) {
				$ids = array_merge($ids, self::getIds($category->getChildren()));
			}
		}
		
		return $ids;
	}
}