<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Filtering Module Helper.
 */
abstract class RSDirectoryFilteringHelper {
	static $selected_categories = array();
	
	public static function getCategories($selected_categories = array()) {
		$options = array(
			'extension' 	=> 'com_rsdirectory',
			'table' 		=> '#__rsdirectory_entries',
			'field' 		=> 'category_id',
			'statefield' 	=> 'published',
			'countItems' 	=> false
		);
			
		// Create a new JCategories object.
		$categories = new JCategories($options);
		
		if ($root = $categories->get('root')) {
			if ($children = $root->getChildren()) {
				$selected_categories = array_filter($selected_categories);

				if (!empty($selected_categories)) {
					self::$selected_categories = $selected_categories;

					self::getIds($children);
					$children = self::remakeChildren($children);
				}

				return $children;
			}
		}
		
		return false;
	}

	protected static function remakeChildren($children) {
		$new_children = array();
		foreach ($children as $child) {
			if(!in_array($child->id, self::$selected_categories)) {
				if ($tmp = $child->getChildren()) {
					$new_children = array_merge($new_children, self::remakeChildren($tmp));
				}
			} else {
				$new_children[] = $child;
			}
		}

		return $new_children;
	}

	protected static function getIds($categories) {
		foreach ($categories as $category) {
			$has_children = $category->hasChildren();
			$children = null;

			// if selected categories are defined search inside
			if (self::$selected_categories) {
				$children_ids = array();

				if ($has_children) {
					$children = $category->getChildren();

					foreach ($children as $child) {
						$children_ids[] = $child->id;
					}

					$intersect = array_intersect(self::$selected_categories, $children_ids);

					// if the child categories are not in the selected categories but the parent is add all the children to the ids
					if (count($intersect) == 0 && in_array($category->id, self::$selected_categories)) {
						$tmp = array_merge(self::$selected_categories, $children_ids);
						self::$selected_categories = array_unique($tmp);
					}
				}
			}
			// if the category has any children parse them too
			if ($has_children) {
				self::getIds((!is_null($children) ? $children : $category->getChildren()));
			}
		}
	}
	
	public static function numEntriesCategories($category_id = null) {
		static $num_items = array();

		// skip for null
		if (is_null($category_id)) {
			return 0;
		}

		if (!isset($num_items[$category_id])) {
			$num_items[$category_id] = RSDirectoryHelper::countEntriesFromCategory($category_id);
		}

		return (int) $num_items[$category_id];
	}
	
    /**
     * Get categories options.
     *
     * @access public
     *
     * @static
     *
     * @param array $categories
     * @param int $level
     *
     * @return array
     */
    public static function outputCategories($categories, $filter_type, $level = 0) {
		$jinput =  JFactory::getApplication()->input;
		$checked_categories = $jinput->get( 'categories', array(), 'array' );

		static $categories_found;

		if (is_null($categories_found))
		{
			$categories_found = array();
		}
		// if a category is selected by the listing add it to the selected ones
		$category = (int) $jinput->get('category', '');

		if (!empty($category))
		{
			$checked_categories[] = (string)$category;
		}
		
		if(empty($categories))
		{
			return $categories_found;
		}

		foreach($categories as $category)
		{
			if ( !$category->published ) continue;

			$sufix = '';
			$prefix = '';

			if ($filter_type == 'checkbox')
			{
				$sufix = '<span class="'.RsdirectoryAdapterGrid::badgeClass('secondary').'">'.self::numEntriesCategories($category->id).'</span>';
			}
			
			$prefix = str_repeat('- ', $level);

			$categories_found[] =  $category->id.'|'.$prefix.htmlspecialchars($category->title, ENT_QUOTES, "utf-8").$sufix.(in_array($category->id, $checked_categories) ? '[c]':'');

			if($subcategories = $category->getChildren())
			{
				self::outputCategories($subcategories, $filter_type, $level + 1);
			}
		}

		return $categories_found;
    }
}