<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! MosetsTree Import Plugin.
 */
class plgRsdirectoryImportMosetsTree extends JPlugin
{
    const IMPORT_OPTION = 'mosetstree';
    const PLUGIN = 'importmosetstree';
    const EXTENSION = 'plg_system_importmosetstree';
    const OPTION = 'PLG_RSDIRECTORY_IMPORTMOSETSTREE_NAME';
	const VERSION = '1.1.0';
	const MINIMUM_MOSETSTREE_VERSION = '3.8.6';
		
	/**
	 * Automatically load language files.
	 *
	 * @access protected
	 *
	 * @var boolean
	 */
	protected $autoloadLanguage = true;
		
	/**
	 * An array of sections that need to be processed.
	 *
	 * @access protected
	 *
	 * @static
	 *
	 * @var array
	 */
	protected static $sections = array(
		0 => 'forms',
		1 => 'categories',
		2 => 'fields',
		3 => 'entries',
		4 => 'reviews',
	);
		
	/**
	 * An array of field types that will imported.
	 *
	 * @access protected
	 *
	 * @static
	 *
	 * @var array
	 */
	protected static $ignore_field_types = array(
		'corerating',
		'corehits',
		'corevotes',
		'corefeatured',
		'coremodified',
		'corevisited',
		'corepublishup',
		'corepublishdown',
		'coreuser',
		'corecreated',
		'coremetakey',
		'coremetadesc',
		'mtags',
		'videoplayer',
		'timezone',
		'termsandconditions',
		'associatedlisting',
		'category',
		'directory',
		'listingid',
		'captcha',
		'vimeo',
		'listings',
		'multipledates',
		'audioplayer2'
	);
		
	/**
	 * An array of MosetsTree image sizes names.
	 *
	 * @access protected
	 *
	 * @static
	 *
	 * @var array
	 */
	protected static $image_sizes = array(
		'o',
		's',
		'm'
	);

    /**
     * Can the plugin run?
     *
     * @access public
     * 
     * @return bool
     */
    public function canRun()
    {
		$app = JFactory::getApplication();
		$db = JFactory::getDbo();
			
		// Get extension data.
		$query = $db->getQuery(true)
		       ->select('*')
			   ->from( $db->qn('#__extensions') )
			   ->where( $db->qn('element') . ' = ' . $db->q('com_mtree') );
			   
		$db->setQuery($query);
			
        return file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php') &&
			   $app->isClient('administrator') && $app->input->get('option') == 'com_rsdirectory' &&
			   $db->loadObject();
    }
     
    /**
     * Add the current import option to the import options list.
     *
     * @access public
     * 
     * @return string
     */
    public function onRsdirectoryaddImportOptions()
    {
		if ( $this->canRun() )
		{
			return JHtml::_( 'select.option', self::IMPORT_OPTION, JText::_(self::OPTION) );
		}
    }
		
	/**
	 * Display import fieldset.
	 *
	 * @access public
	 */	
	public function onRsdirectoryDisplayImportFieldset()
	{
		if ( !$this->canRun() )
			return;

		JHtml::_('script', 'plg_rsdirectory_importmosetstree/script.js', array('relative' => true, 'version' => 'auto'));
			
		$has_pce = self::hasEntriesStoredInParentCategories();
		$has_mce = self::hasEntriesStoredInMultipleCategories();

		JForm::addFormPath(__DIR__ . '/forms');

		$form = JForm::getInstance( 'plg_rsdirectory_importmosetstree.importoptions', 'importoptions', array('control' => 'jform'));

		$pce_fields = array('pce', 'pce_rsdirectory_category', 'pce_mosetstree_category');
		$mce_fields = array('mce', 'mce_rsdirectory_category', 'mce_mosetstree_category');

		// remove and add conditions to the fields
		foreach ($form->getFieldsets() as $fieldset)
		{
			if ($fields = $form->getFieldset($fieldset->name))
			{
				foreach ($fields as $field)
				{
					// if there is no pce do not render them
					if (in_array($field->fieldname, $pce_fields) && !$has_pce)
					{
						$form->removeField($field->fieldname, 'mosetstree');
					}

					// if there is no pce and no mce do not render the mce fields
					if (in_array($field->fieldname, $mce_fields) && !$has_pce && !$has_mce)
					{
						$form->removeField($field->fieldname, 'mosetstree');
					}

					// if pce are present but no mce the only case that mce must be displayed is for pce:lowest_subcategories option
					if (in_array($field->fieldname, $mce_fields) && $has_pce && !$has_mce)
					{
						$current_showon = $form->getFieldAttribute($field->fieldname, 'showon', null, 'mosetstree');
						$new_showon = 'pce:lowest_subcategories';
						if (!empty($current_showon))
						{
							$new_showon .='[AND]'.$current_showon;
						}

						$form->setFieldAttribute($field->fieldname, 'showon', $new_showon, 'mosetstree');
					}
				}
			}
		}

		?>

		<fieldset id="mosetstree" class="import-fieldset hide">
			<?php
				foreach ($form->getFieldsets() as $fieldset)
				{
					if ($fields = $form->getFieldset($fieldset->name))
					{
						foreach ($fields as $field)
						{
							echo $field->renderField();
						}
					}
				}
			?>
		</fieldset>
			
		<?php
			
		JText::script('PLG_RSDIRECTORY_IMPORTMOSETSTREE_REQUIRED_FIELDS_ERROR');
	}
		
	/**
	 * Import.
	 *
	 * @access public
	 *
	 * @param object $vars
	 */
	public function onRsdirectoryImport($vars)
	{
		// Do a few checks.
		if ( !$this->canRun() || empty($vars->import_from) || $vars->import_from != self::IMPORT_OPTION || !$vars->import_action )
			return;
			
		switch ($vars->import_action)
		{
			case 'verify':
					
				$this->verify($vars);
				break;
					
			case 'measure':
					
				$this->measure($vars);
				break;
					
			case 'import':
					
				$this->import($vars);
				break;
		}
	}
		
	/**
	 * Method to verify import data.
	 *
	 * @access protected
	 *
	 * @param object $vars
	 */

	protected function verify($vars)
	{
		$messages = $vars->response['messages'];
			
		$db = JFactory::getDbo();
			
		// Get extension data.
		$query = $db->getQuery(true)
		       ->select('*')
			   ->from( $db->qn('#__extensions') )
			   ->where( $db->qn('element') . ' = ' . $db->q('com_mtree') );
			   
		$db->setQuery($query);
			
		$extension = $db->loadObject();
			
		if ( empty($extension) )
		{
			$messages[] = array(
				'message' => JText::_('PLG_RSDIRECTORY_IMPORTMOSETSTREE_COMPONENT_NOT_INSTALLED'),
				'type' => 'error',
				'id' => null,
			);
		}
		else
		{
			$messages[] = array(
				'message' => JText::_('PLG_RSDIRECTORY_IMPORTMOSETSTREE_COMPONENT_INSTALLED'),
				'type' => 'success',
				'id' => null,
			);
				
			$xmlPath = JPATH_ADMINISTRATOR.'/components/com_mtree/mtree.xml';
			
			$version = false;
			if (file_exists($xmlPath) && ($xml = file_get_contents($xmlPath))) {
				if (preg_match('#<version>(.*?)<\/version>#', $xml, $match)) {
					$version = $match[1];
				}
			} elseif (!empty($extension->manifest_cache) && ($manifest_cache = json_decode($extension->manifest_cache)) && isset($manifest_cache->version)) {
				$version = $manifest_cache->version;
			}
				
			if ( empty($version) )
			{
				$messages[] = array(
					'message' => JText::_('PLG_RSDIRECTORY_IMPORTMOSETSTREE_UNKNOWN_VERSION'),
					'type' => 'error',
					'id' => null,
				);
			}
			else if ( version_compare(self::MINIMUM_MOSETSTREE_VERSION, $version) > 0 )
			{
				$messages[] = array(
					'message' => JText::sprintf('PLG_RSDIRECTORY_IMPORTMOSETSTREE_VERSION_INCOMPATIBLE', $version),
					'type' => 'error',
					'id' => null,
				);
			}
			else
			{
				$messages[] = array(
					'message' => JText::sprintf('PLG_RSDIRECTORY_IMPORTMOSETSTREE_VERSION_OK', $version),
					'type' => 'success',
					'id' => null,
				);
					
				$vars->response['action'] = 'measure';
			}
		}
			
		$vars->response['messages'] = $messages;
	}
		
	/**
	 * Method to measure import data.
	 *
	 * @access protected
	 *
	 * @param object $vars
	 */

	public function measure($vars)
	{
		// Get DBO.
		$db = JFactory::getDbo();
			
		$messages = $vars->response['messages'];
			
		$totals = array();
			
		$total = 0;
			
		foreach (self::$sections as $section)
		{
			if ($section == 'forms')
			{
				$count = $this->getMosetsTreeForms(true);
				$totals[$section] = $count;
				$total += $count;
				
				continue;
			}
			else if ($section == 'categories')
			{
				$query = $db->getQuery(true)
					   ->select('COUNT(*)')
					   ->from( $db->qn('#__mt_cats'))
					   ->where( $db->qn('cat_id').' != '.$db->q(0));
			}
			else if ($section == 'fields')
			{
				$query = $db->getQuery(true)
					   ->select('COUNT(*)')
					   ->from( $db->qn('#__mt_customfields') )
					   ->where( $db->qn('field_type') . ' NOT IN (' . RSDirectoryHelper::quoteImplode(self::$ignore_field_types) . ')' );
					   
			}
			else if ($section == 'entries')
			{
				$query = $db->getQuery(true)
					   ->select('COUNT(*)')
					   ->from( $db->qn('#__mt_links') );
			}

			else if ($section == 'reviews')
			{
				$query = $db->getQuery(true)
					->select('COUNT(*)')
					->from( $db->qn('#__mt_reviews') );
			}
			
				
			$db->setQuery($query);
			$count = (int) $db->loadResult();

			if ($section == 'fields' && $this->checkForMapField()){
				$count++;
			}
				
			$totals[$section] = $count;
			$total += $count;
		}
			
		$totals['total'] = $total;
			
		JFactory::getApplication()->setUserState('com_rsdirectory.measure.import.totals', $totals);
			
		$messages[] = array(
			'message' => JText::_('COM_RSDIRECTORY_IMPORT_MEASURING_DATA_DONE'),
			'type' => 'success',
			'id' => 'import-msg-data-measuring',
		);
			
		$messages[] = array(
			'message' => JText::_('COM_RSDIRECTORY_IMPORTING_DATA'),
			'type' => 'info',
			'id' => null,
		);
			
		$section = self::$sections[0];
			
		$messages[] = array(
			'message' => JText::sprintf('COM_RSDIRECTORY_IMPORT_SECTION_PROGRESS', $section, 0.0),
			'type' => 'success',
			'id' => self::sectionNameToId($section),
		);
			
		$vars->response['action'] = 'import';
		$vars->response['section'] = $section;
		$vars->response['messages'] = $messages;
	}

	/**
	 * Method to get the forms from MosetsTree (number or fields)
	 *
	 * @access protected
	 */

	protected function getMosetsTreeForms($only_count = false){
		// Get DBO.
		$db = JFactory::getDbo();

		$query = $db->getQuery(true);
		$query->select('DISTINCT('.$db->qn('cat_id').')')
			  ->from($db->qn('#__mt_fields_map'))
			  ->where($db->qn('cat_id') .' != '. $db->q(0));

		$db->setQuery($query);
		$return_data = array();

		$cats_with_same_hash = array();
		if ($cats = $db->loadColumn()) {
			foreach ($cats as $cat_id) {
				$query = $db->getQuery(true);
				$query->select($db->qn('cf_id'))
					  ->from($db->qn('#__mt_fields_map'))
					  ->where($db->qn('cat_id') .'='.$db->q($cat_id));
				$db->setQuery($query);

				$fields = $db->loadColumn();
				$hash = implode('', $fields);
				$hash = md5($hash);
				$return_data[] = array('fields'=>$fields, 'hash' => $hash);
				if (!isset($cats_with_same_hash[$hash])) {
					$cats_with_same_hash[$hash] = array();
				}
				$cats_with_same_hash[$hash][] = $cat_id;
			}
		}

		if ($only_count) {
			$forms_found = array();
			foreach ($return_data as $data) {
				if (!in_array($data['hash'], $forms_found)) {
					$forms_found[] = $data['hash'];
				}
			}

			return count($forms_found);

		} else {
			// group all the categories for the same hashes
			foreach($return_data as $index => $data) {
				if (isset($cats_with_same_hash[$data['hash']])) {
					$return_data[$index]['cat_ids'] =  $cats_with_same_hash[$data['hash']];
				}
			}
			
			$forms_found = array();
			foreach ($return_data as $data) {
				$forms_found[$data['hash']] = $data;
			}
			
			$forms_found = array_values($forms_found);

			return $forms_found;
		}
	}

	protected function getTheFirstParentCategory($cat_id, $sections){

		if (isset($sections[$cat_id])) return $sections[$cat_id];
		// Get DBO.
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
			->select( $db->qn('cat_parent') )
			->from( $db->qn('#__mt_cats') )
			->where( $db->qn('cat_id') .' = '.$db->q($cat_id) )
			->where( $db->qn('cat_parent') .' != '.$db->q(0) );

		$db->setQuery($query);

		$parent_cat = $db->loadResult();
		if (!empty($parent_cat)){
			$parent_cat = $this->getTheFirstParentCategory($parent_cat, $sections);
		} else {
			return 0;
		}

		return $parent_cat;
	}
		
	/**
	 * Method to import data.
	 *
	 * @access protected
	 *
	 * @param object $vars
	 */

	protected function checkForMapField() {
		// Get DBO.
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
			->select('COUNT(*)')
			->from( $db->qn('#__mt_links') )
			->where( $db->qn('show_map') .'='. $db->q(1))
			->where( $db->qn('lat') .'>'. $db->q(0))
			->where( $db->qn('lng') .'>'. $db->q(0));

		$db->setQuery($query);
		return (int) $db->loadResult();
	}

	protected function import($vars)
	{
		// Get application.
		$app = JFactory::getApplication();
			
		// Get form data.
		$jform_data = $app->input->get( 'jform', array(), 'array' );
			
		// Get plugin form data.
		$data = empty($jform_data[self::IMPORT_OPTION]) ? array() : $jform_data[self::IMPORT_OPTION];
			
		$section = empty($data['section']) ? '' : $data['section'];
		$section_offset = empty($data['section_offset']) ? 0 : $data['section_offset'];
		$offset = empty($data['offset']) ? 0 : $data['offset'];
		
		if ( !$section || !in_array($section, self::$sections) )
		{
			$vars->response['messages'][] = array(
				'message' => JText::_('COM_RSDIRECTORY_INVALID_DATA_PROVIDED'),
				'type' => 'error',
				'id' => null,
			);
				
			return;
		}

		// Get DBO.
		$db = JFactory::getDbo();
			
		$messages = $vars->response['messages'];
		$tmp_values = array();
			
		// Empty the import temporary table.
		if (!$offset)
		{
			$db->setQuery( 'TRUNCATE TABLE ' . $db->qn('#__rsdirectory_import_tmp') );
			$db->execute();
		}
			
		$limit = 100;

		$user = JFactory::getUser();
			
		if ($section == 'forms')
		{
			$mosets_forms_all = $this->getMosetsTreeForms(false);
			$mosets_forms_all = array_chunk($mosets_forms_all, $limit, true);
			$mosets_form_index = ($section_offset/$limit);
			$objects = $mosets_forms_all[$mosets_form_index];

			if (!empty($objects))
			{
				$form_model = RSDirectoryModel::getInstance( 'Form', 'RSDirectoryModel', array('ignore_request' => true) );

				$processed_forms = (!empty($data['$processed_forms']) ? $data['$processed_forms'] : array());
				foreach ($objects as $form_data)
				{
					if (!in_array($form_data['hash'], $processed_forms)) {
						$data = array();

						if ($form = $form_model->getForm()) {
							$fieldsets = $form->getFieldsets();

							foreach ($fieldsets as $name => $fieldset) {
								$fields = $form->getFieldset($name);

								// Set properties with their default values.
								foreach ($fields as $field) {
									$data[$field->fieldname] = $form->getFieldAttribute($field->fieldname, 'default');
								}
							}
						}

						$data['title'] = JText::_('PLG_RSDIRECTORY_IMPORTMOSETSTREE_GENERATED_FORM').(count($processed_forms) + 1);

						$form_model->save($data);

						$new_form_id = $form_model->getState($form_model->getName() . '.id');

						foreach ($form_data['cat_ids'] as $cat_id) {
							$tmp = array(
								$db->q($cat_id),
								$db->q($new_form_id),
								$db->q($section),
							);

							$tmp_values[] = implode(',', $tmp);
						}

						$processed_forms[] = $form_data['hash'];
					}
				}

				$vars->processed_forms = $processed_forms;
			}
		}
		else if ($section == 'categories')
		{
			$limit = 60;

			$query = $db->getQuery(true)
				->select( '*' )
				->from( $db->qn('#__mt_cats') )
				->where( $db->qn('cat_id') .' != '.$db->q(0) )
				->order( $db->qn('ordering') );
				
			$db->setQuery($query, $section_offset, $limit);

			$objects = $db->loadObjectList();
				
			if ($objects)
			{
				$category_model = RSDirectoryModel::getInstance( 'Category', 'RSDirectoryModel', array('ignore_request' => true) );
					
				$parents = $this->getTmp('categories');
				$sections = $this->getTmp('forms');

				foreach ($objects as $object)
				{
					$file_id = 0;

					$object->cat_id = (int) $object->cat_id;
						
					// Process thumbnail.
					if ($object->cat_image)
					{
						// Set the source file.
						$src = realpath(JPATH_ROOT . "/media/com_mtree/images/cats/o/$object->cat_image");
							
						// Get the extension.
						$ext = strtolower( JFile::getExt($object->cat_image) );
							
						// Get the hash.
						$hash = RSDirectoryHelper::getHash();
							
						// Generate a file name.
						$file_name = $hash . ($ext ? ".$ext" : '');
							
						// Set the destination file.
						$dest = JPATH_ROOT . "/components/com_rsdirectory/files/images/categories/$file_name";

						if ( JFile::copy($src, $dest) )
						{
							$thumb = (object)array(
								'user_id' => $user->id,
								'hash' => $hash,
								'file_name' => $file_name,
								'original_file_name' => basename($object->cat_image),
							);
								
							$db->insertObject('#__rsdirectory_uploaded_files', $thumb, 'id');
								
							// Get the file id.
							$file_id = $thumb->id;
						}
					}
						
					$parent_id =  !isset($parents[$object->cat_parent]) ? 1 : $parents[$object->cat_parent];
					list($title, $alias) = self::getTitleAndAlias($object->cat_name, $parent_id);

					$form_id = $this->getTheFirstParentCategory($object->cat_id, $sections);
					$params = array(
						'form_id' => $form_id,
					);
						
					$data = array(
						'parent_id' => $parent_id,
						'title' => $title,
						'alias' => $alias,
						'description' => stripcslashes($object->cat_desc),
						'metadesc' => $object->metadesc,
						'metakey' => $object->metakey,
						'metadata' => array(
							'author' => '',
							'robots' => '',
						),
						'params' => $params,
						'published' => $object->cat_published == 1 ? 1 : 0,
						'created_time' => $object->cat_created,
						'created_user_id' => $user->id,
						'modified_time' => '',
						'modified_user_id' => 0,
						'hits' => 0,
						'language' => '*',
					);
					
					// Save the category.
					if (!$category_model->save($data)) {
						$vars->response['messages'][] = array(
							'message' => JText::sprintf('PLG_RSDIRECTORY_IMPORTMOSETSTREE_CATEGORY_SAVE_ERROR', $title, $category_model->getError()),
							'type' => 'error',
							'id' => null,
						);
						
						return;
					}
						
					// Get the id of the newly inserted category.
					$category_id = $category_model->getState( $category_model->getName() . '.id' );
						
					// Reset the category id.
					$category_model->setState( $category_model->getName() . '.id', null );
						
					// Insert the file - category relation.
					if ($file_id)
					{
						$relation = (object)array(
							'file_id' => $file_id,
							'category_id' => $category_id,
						);
							
						$db->insertObject('#__rsdirectory_uploaded_files_categories_relations', $relation);
							
						// Load params.
						$query = $db->getQuery(true)
							   ->select( $db->qn('params') )
							   ->from( $db->qn('#__categories') )
							   ->where( $db->qn('id') . ' = ' . $db->q($category_id) );
							   
						$db->setQuery($query);
						$params = (array)json_decode( $db->loadResult() );
						$params['thumbnail'] = $file_id;
							
						$category = (object)array(
							'id' => $category_id,
							'params' => json_encode($params),
						);
							
						$db->updateObject('#__categories', $category, 'id');
					}
						
					$parents[$object->cat_id] = $category_id;

					$query = $db->getQuery(true)
						->select( 'cat_id' )
						->from( $db->qn('#__mt_cats') )
						->where( $db->qn('cat_parent') . ' = ' . $db->q($object->cat_id) );

					$db->setQuery($query);
						
					if ( $children_old_ids = $db->loadColumn() )
					{
						$children_new_ids = self::getNewIds($parents, $children_old_ids);
							
						foreach ($children_new_ids as $child_new_id)
						{
							$category_update = (object)array(
								'id' => $child_new_id,
								'parent_id' => $category_id,
							);
								
							$db->updateObject('#__categories', $category_update, 'id');
						}
							
						// Rebuild.
						$category_model->rebuild();
					}
						
					$tmp = array(
						$db->q($object->cat_id),
						$db->q($category_id),
						$db->q($section),
					);
						
					$tmp_values[] = implode(',', $tmp);	
				}
			}
		}
		else if ($section == 'fields')
		{
			$query = $db->getQuery(true)
					->select('*')
					->from( $db->qn('#__mt_customfields') )
					->where( $db->qn('field_type') . ' NOT IN (' . RSDirectoryHelper::quoteImplode(self::$ignore_field_types) . ')' );

			$db->setQuery($query, $section_offset, $limit);
			$objects = $db->loadObjectList();

			$isMapAdded = (int) $app->getUserState('com_rsdirectory.measure.import.ismapadded');
			if ($this->checkForMapField() && !$isMapAdded){
				JFactory::getApplication()->setUserState('com_rsdirectory.measure.import.ismapadded', 1);

				$map_field = new stdClass();
				$map_field->field_type = 'map';
				$map_field->alias = 'map';
				$map_field->published = 1;
				$map_field->required_field = 0;
				$map_field->params = '';
				$map_field->simple_search = 1;
				$map_field->advanced_search = 0;
				$map_field->size = 0;
				$map_field->default_value = '';
				$map_field->field_elements = '';
				$map_field->details_view = 1;
				$map_field->cf_id = 0;

				$objects[] = $map_field;
			}
				
			if ($objects)
			{
				$field_model = RSDirectoryModel::getInstance( 'Field', 'RSDirectoryModel', array('ignore_request' => true) );
					
				$sections = $this->getTmp('forms');
					
				$query = $db->getQuery(true)
					   ->select( $db->qn('id') . ', ' . $db->qn('type') )
					   ->from( $db->qn('#__rsdirectory_field_types') );
					   
				$db->setQuery($query);
				$field_types = $db->loadObjectList();
					
				$field_types_ids = array();

				foreach ($objects as $object) {
					$rsdirectory_field_type = $this->correlateFieldType($object->field_type);
					if ( $field_type = RSDirectoryHelper::findElements( array('type' => $rsdirectory_field_type), $field_types ) )
					{
						$field_types_ids[$object->field_type] = $field_type->id;
					}
				}
					
				// Initialize the custom fields values array.
				$custom_fields = array();
					
				foreach ($objects as $object)
				{

					// let's get the forms that this field is in
					$forms_ids = array();

					if ($object->cf_id) {
						$query = $db->getQuery(true)
							->select($db->qn('cat_id'))
							->from($db->qn('#__mt_fields_map'))
							->where($db->qn('cf_id') . ' = ' . $db->q($object->cf_id));

						$db->setQuery($query);


						if ($cats = $db->loadColumn()) {
							foreach ($cats as $cat_id) {
								if (!empty($sections[$cat_id])) {
									$forms_ids[] = $sections[$cat_id];
								}
							}
						}

						$forms_ids = array_unique($forms_ids);
					}

					// Skip the description field type, because it's a core field
					if ( in_array($object->field_type, array('coredesc', 'coreprice'))) {

						$look_for = '';
						if ($object->field_type == 'coredesc') $look_for = 4;
						if ($object->field_type == 'coreprice') $look_for = 6;

						if (empty($look_for)) continue;
						// add the description and price to the form, but don't save them as a filed
						$query = $db->getQuery(true)
							->select($db->qn('id'))
							->select($db->qn('field_type_id'))
							->from($db->qn('#__rsdirectory_fields'))
							->where($db->qn('field_type_id').' = '.$db->q($look_for));
						$db->setQuery($query);

						if ($found_field = $db->loadObject()) {
							$query = $db->getQuery(true)
								->select( $db->qn('form_id') )
								->from( $db->qn('#__rsdirectory_forms_fields') )
								->where( $db->qn('field_id') . ' = ' . $db->q($found_field->id) );

							$db->setQuery($query);
							$forms = $db->loadColumn();

							if ( $add = array_diff(RSDirectoryHelper::arrayInt($forms_ids), $forms) )
							{
								foreach ($add as $form_id)
								{
									$field_model->assign2Form($found_field->id, $form_id);
								}
							}
						}

						continue;
					}
						
					// Initialize the data array.
					$data = array();
						
					if ( isset($field_types_ids[$object->field_type]) )
					{
						$app->input->set('field_type_id', $field_types_ids[$object->field_type]);
							
						if ( $form = $field_model->getForm() )
						{
							$fieldsets = $form->getFieldsets();
								
							foreach ($fieldsets as $name => $fieldset)
							{
								$fields = $form->getFieldset($name);
									
								// Set properties with their default values.
								foreach ($fields as $field)
								{
									$data[$field->fieldname] = $form->getFieldAttribute($field->fieldname, 'default');
								}
							}
						}
					}

					if (!empty($object->alias)) {
						$name = str_replace('_', '-', $object->alias);
					} else {
						$name = stripslashes($object->caption);
						$name = JFilterOutput::stringURLSafe(trim($name));

						if ( trim( str_replace('-', '', $name) ) == '' )
						{
							$name = $object->field_type;
						}
					}

					$name_tmp = $name;
						
					$i = 2;
						
					do
					{
						$query = $db->getQuery(true)
							   ->select( $db->qn('id') )
							   ->from( $db->qn('#__rsdirectory_fields') )
							   ->where( $db->qn('name') . ' = ' . $db->q($name_tmp) );
								
						$db->setQuery($query);
							
						// Check if the name property value is unique.
						if ( $result = $db->loadResult() )
						{
							$name_tmp = $name . $i;
								
							$i++;
						}
					}
					while ($result);



					$data['name'] = $name_tmp;
					$data['field_type_id'] = $field_types_ids[$object->field_type];


					if (!empty($forms_ids)) {
						$data['forms'] = $forms_ids;
					}

					// end getting the forms that the field is in

					$data['published'] = $object->published;
					$data['required'] = $object->required_field;
						
					if ( isset($object->caption) )
					{
						$object->name = stripslashes($object->caption);
						self::updateFieldProperty('form_caption', $object->caption, $data);
						self::updateFieldProperty('listing_caption', $object->caption, $data);
						self::updateFieldProperty('searchable_advanced_caption', $object->caption, $data);
					}
						
					if ( isset($object->prefix_text_mod) )
					{
						self::updateFieldProperty('field_prepend', $object->prefix_text_mod, $data);
					}

					if ( isset($object->suffix_text_mod) )
					{
						self::updateFieldProperty('field_append', $object->suffix_text_mod, $data);
					}
						
					$filter = 'none';
					$regex_syntax = '';

						
					switch ($object->field_type)
					{
						case 'mnumber':

							$filter = 'numeric';
							break;

						case 'coreemail':
						case 'memail':

							$filter = 'email';
							break;

						case 'coremobile':
						case 'corefax':

							$filter = 'regex';
							$regex_syntax = '/^(\+\d{1,3}\s?)?(\s?\([\d]\)\s?)?[\d\-\s\.]+$/';
							break;
					}
						
					self::updateFieldProperty('default_validation_rule', $filter, $data);
					self::updateFieldProperty('regex_syntax', $regex_syntax, $data);

					// there are no credits
					self::updateFieldProperty('credits', 0, $data);


					// get field params
					$object->params = explode("\n", $object->params);
					$params = array();
					foreach ($object->params as $param) {
						if (!empty($param)) {
							$param = explode("=", $param);
							$params[$param[0]] = $param[1];
						}
					}
						
					$searchable_simple = $object->simple_search;
					$searchable_advanced = ($object->advanced_search ? 'textbox' : '');
					$searchable_advanced_condition_type = 'strict';
					$use_field_items = 1;

						
					if ( $object->size )
					{
						self::updateFieldProperty('size', $object->size, $data);
					}
						
					if ( $object->default_value )
					{
						if ( in_array( $object->field_type, array('mcheckbox', 'mselectmultiple', 'selectlist') ) )
						{
							self::updateFieldProperty('default_values',$object->default_value, $data);
						}
						else
						{
							self::updateFieldProperty('default_value',$object->default_value, $data);
						}
					}
						
					// Convert the maximum file size from bytes to kilobytes and update field property.
					if ( isset($params['maxSize']) )
					{
						self::updateFieldProperty('maximum_file_size', $params['maxSize'] / 1024, $data);
					}
						
					self::updateFieldProperty('max_files_number', 1, $data);
						
					if ( $object->field_type == 'textarea' )
					{
						if ( !empty($searchable_simple)  || !empty($searchable_advanced))
						{
							$searchable_advanced = 'textbox';
							$searchable_advanced_condition_type = 'containing';
						}
					}
						
					self::updateFieldProperty('searchable_simple', $searchable_simple, $data);
					self::updateFieldProperty('searchable_advanced', $searchable_advanced, $data);
					self::updateFieldProperty('use_dependency', 0, $data);
					self::updateFieldProperty('use_field_items', $use_field_items, $data);
					self::updateFieldProperty('searchable_advanced_condition_type', $searchable_advanced_condition_type, $data);

					self::updateFieldProperty('allow_html', $object->field_type == 'texteditor' ? 1 : 0, $data);


					if ( $object->field_elements)
					{

						$object->field_elements = str_replace('|', "\n", $object->field_elements);
						$field_options = explode("\n", $object->field_elements);
							
						self::updateFieldProperty( 'items', implode("\n", $field_options), $data );
					}

					if ($object->field_type == 'mselectmultiple')
					{
						self::updateFieldProperty('multiple', 1, $data);
					}
						
					if ( $field_model->save($data) )
					{
						$field_id = $field_model->getState( $field_model->getName() . '.id' );
							
						// Build custom fields for detail view.
						if ( $object->field_type != 'image' &&  $object->details_view)
						{
							foreach ($forms_ids as $form_id) {
								$custom_fields[] = array(
									$db->q($form_id),
									$db->q($field_id),
								);
							}
						}

						if ($object->cf_id) {
							$tmp = array(
								$db->q($object->cf_id),
								$db->q($field_id),
								$db->q($section),
							);

							$tmp_values[] = implode(',', $tmp);
						}
					}
				}
					
				if ($custom_fields)
				{
					// Insert columns.
					$columns = array('form_id', 'field_id');
						
					// Insert the custom form fields in the database.
					$query = $db->getQuery(true)
						   ->insert( $db->qn('#__rsdirectory_forms_custom_fields') )
						   ->columns( $db->qn($columns) );
							
					foreach ($custom_fields as $field)
					{
						$query->values( implode(',', $field) );
					}

					$db->setQuery($query);
					$db->execute();
				}
			}
		}
		else if ($section == 'entries')
		{
			$query = $db->getQuery(true)
				   ->select('*')
				   ->from( $db->qn('#__mt_links') );
				   
			$db->setQuery($query, $section_offset, $limit);
			$objects = $db->loadObjectList();
				
			if ($objects)
			{
				$has_pce = self::hasEntriesStoredInParentCategories();
					
				$pce = empty($data['pce']) ? '' : $data['pce'];
				$pce_rsdirectory_category = empty($data['pce_rsdirectory_category']) ? '' : $data['pce_rsdirectory_category'];
				$pce_mosetstree_category = empty($data['pce_mosetstree_category']) ? '' : $data['pce_mosetstree_category'];
				$mce = empty($data['mce']) ? '' : $data['mce'];
				$mce_rsdirectory_category = empty($data['mce_rsdirectory_category']) ? '' : $data['mce_rsdirectory_category'];
				$mce_mosetstree_category = empty($data['mce_mosetstree_category']) ? '' : $data['mce_mosetstree_category'];

				$entry_model = RSDirectoryModel::getInstance( 'Entry', 'RSDirectoryModel', array('ignore_request' => true) );
					
				$parents = $this->getTmp('categories');
				$fields_tmp = $this->getTmp('fields');
					
				// Get all the form fields.
				$form_fields = RSDirectoryHelper::getFormFields();
					
				// Get entry - categories relations.
				$query = $db->getQuery(true)
				       ->select( $db->qn( array('link_id', 'cat_id') ) )
					   ->from( $db->qn('#__mt_cl') );
					   
				$db->setQuery($query);
				$results = $db->loadObjectList();
					
				$relations = array();
					
				if ($results)
				{
					foreach ($results as $result)
					{
						if ( !isset($relations[$result->link_id]) )
						{
							$relations[$result->link_id] = array();
						}
							
						$relations[$result->link_id][] = $result->cat_id;
					}
				}
					
				// Initialize files array.
				$files = array();
					
				// Process each entry.
				foreach ($objects as $object)
				{
					$categories = array();

					// set the object user id if it's not present
					$default_user_id = $user->id;
					if (!$user->load($object->user_id)) {
						$object->user_id = $default_user_id;
					}
						
					// Check if the entry is in a parent category.
					if ( isset($relations[$object->link_id]) )
					{
						foreach ($relations[$object->link_id] as $category_id)
						{
							if ( $has_pce && self::hasChildren($category_id) )
							{
								if ($pce == 'lowest_subcategories')
								{
									if ( $lowest_subcategories = self::getLowestSubcategories($category_id) )
									{
										$lowest_subcategories = self::getTmpIds($lowest_subcategories, $parents);
										$categories = array_merge($categories, $lowest_subcategories);
									}
								}
								else if ($pce == 'rsdirectory_category')
								{
									$categories[] = $pce_rsdirectory_category;
								}
								else if ($pce == 'mosetstree_category')
								{
									$categories[] = self::getTmpIds($pce_mosetstree_category, $parents);
								}
								else if ($mce == 'ignore_entry')
								{
									// Ignore entry.
									continue 2;
								}
							}
							else
							{
								$categories[] = self::getTmpIds($category_id, $parents);
							}
						}
					}
						
					// Remove duplicates.
					$categories = array_unique($categories);
						
					// Check if the entry is in multiple categories.
					if ( count($categories) > 1 )
					{
						if ($mce == 'first_category')
						{
							$categories = array($categories[0]);
						}
						else if ($mce == 'rsdirectory_category')
						{
							$categories = array($mce_rsdirectory_category);
						}
						else if ($mce == 'mosetstree_category')
						{
							$categories = array( self::getTmpIds($mce_mosetstree_category, $parents) );
						}
						else if ($mce == 'ignore_entry')
						{
							// Ignore entry.
							continue;
						}
					}
						
					if ($categories)
					{
						// Initialize entry data array.
						$data = array(
							'title' => $object->link_name,
							'description' => $object->link_desc,
							'price' => $object->price,
							'promoted' => $object->link_featured,
							'user_id' => $object->user_id,
							'created_time' => $object->link_created,
							'published_time' => $object->publish_up,
							'modified_user_id' => $object->user_id,
							'modified_time' => $object->link_modified,
							'expiry_time' => $object->publish_down,
							'published' => $object->link_published,
							'charge' => 0,
							'import' => 1,
						);
							
						// Get MosetsTree form data.
						$select = array(
							$db->qn('cfv') . '.*',
							$db->qn('f.field_type'),
						);
							
						$query = $db->getQuery(true)
							   ->select($select)
							   ->from( $db->qn('#__mt_cfvalues', 'cfv') )
							   ->innerJoin( $db->qn('#__mt_customfields', 'f') . ' ON ' . $db->qn('cfv.cf_id') . ' = ' . $db->qn('f.cf_id') )
							   ->where( $db->qn('f.field_type') . ' NOT IN (' . RSDirectoryHelper::quoteImplode(self::$ignore_field_types) . ')' )
							   ->where( $db->qn('cfv.attachment') . ' != '. $db->q(1)  )
							   ->where( $db->qn('cfv.link_id') . ' = ' . $db->q($object->link_id) );
								
						$db->setQuery($query);
							
						// Add MosetsTree data to entry data array.
						if ( $items = $db->loadObjectList() )
						{
							foreach ($items as $item)
							{
								// Skip the form field if it wasn't imported into RSDirectory!
								if ( empty($fields_tmp[$item->cf_id]) )
									continue;
									
								// Get the RSDirectory! form field id.
								$field_id = $fields_tmp[$item->cf_id];
									
								// Find the form field.
								$form_field = RSDirectoryHelper::findElements( array('id' => $field_id), $form_fields );
									
								// Skip the form field if it wasn't found in the RSDirectory! fields list.
								if ( empty($form_field) )
									continue;
									
								if ( $form_field->create_column && !isset($data[$form_field->form_field_name]) )
								{
									$field_value = stripslashes($item->value);

									if ($form_field->properties->get('multiple')) {
										$field_value = explode('|', $field_value);
									}
									$data[$form_field->form_field_name] = $field_value;
								}
							}
						}

						// handle core fields that are allowed
						$query = $db->getQuery(true)
							->select(array(
								$db->qn('f.field_type'),
								$db->qn('f.cf_id'),
							))
							->from( $db->qn('#__mt_customfields', 'f') )
							->where( $db->qn('f.field_type') . ' NOT IN (' . RSDirectoryHelper::quoteImplode(self::$ignore_field_types) . ')' )
							->where( $db->qn('f.field_type') . ' LIKE "core%"');

						$db->setQuery($query);

						// Add MosetsTree core data (like city state etc.) to entry data array.
						if ( $items = $db->loadObjectList() )
						{
							foreach ($items as $item)
							{
								// Skip the form field if it wasn't imported into RSDirectory!
								if ( empty($fields_tmp[$item->cf_id]) )
									continue;

								// Get the RSDirectory! form field id.
								$field_id = $fields_tmp[$item->cf_id];

								// Find the form field.
								$form_field = RSDirectoryHelper::findElements( array('id' => $field_id), $form_fields );

								// Skip the form field if it wasn't found in the RSDirectory! fields list.
								if ( empty($form_field) )
									continue;

								$without_core_name = str_replace('core', '', $item->field_type);
								if ( $form_field->create_column && !isset($data[$form_field->form_field_name]) && isset($object->{$without_core_name}) && strlen($object->{$without_core_name}))
								{
									$field_value = stripslashes($object->{$without_core_name});

									if ($form_field->properties->get('multiple')) {
										$field_value = explode('|', $field_value);
									}
									$data[$form_field->form_field_name] = $field_value;
								}
							}
						}
							
						// Get MosetsTree images data.
						$query = $db->getQuery(true)
							->select($db->qn('cfva') . '.*')
							->select($db->qn('f.field_type'))
							->select($db->q(0).' as isCore')
							->from( $db->qn('#__mt_cfvalues_att', 'cfva') )
							->innerJoin( $db->qn('#__mt_customfields', 'f') . ' ON ' . $db->qn('cfva.cf_id') . ' = ' . $db->qn('f.cf_id') )
							->where( $db->qn('cfva.link_id') . ' = ' . $db->q($object->link_id) )
							->where( $db->qn('f.field_type') . ' = ' . $db->q('image') );

						$db->setQuery($query);

						$mosetstree_image_fields = $db->loadObjectList();

						// Get core images for the entriers
						$query = $db->getQuery(true)
							->select($db->qn('filename'))
							->select($db->qn('filename', 'raw_filename'))
							->select($db->q(1).' as isCore')
							->from( $db->qn('#__mt_images') )
							->where( $db->qn('link_id') . ' = ' . $db->q($object->link_id) )
							->order($db->qn('ordering'));

						$db->setQuery($query);

						$mosetstree_core_images = $db->loadObjectList();

						// we need to merge all the images so that we can upload them in a single place
						$mosetstree_image_fields = (object) array_merge((array)$mosetstree_image_fields, (array) $mosetstree_core_images);

						// Get MosetsTree files data.
						$query = $db->getQuery(true)
							->select($db->qn('cfva') . '.*')
							->select($db->qn('f.field_type'))
							->from( $db->qn('#__mt_cfvalues_att', 'cfva') )
							->innerJoin( $db->qn('#__mt_customfields', 'f') . ' ON ' . $db->qn('cfva.cf_id') . ' = ' . $db->qn('f.cf_id') )
							->where( $db->qn('cfva.link_id') . ' = ' . $db->q($object->link_id) )
							->where( $db->qn('f.field_type') . ' = ' . $db->q('mfile') );

						$db->setQuery($query);

						$mosetstree_file_uploads = $db->loadObjectList();
							
						// Insert the entry in each category.
						foreach ($categories as $category_id)
						{
							// Get the form associated with the category.
							$form = RSDirectoryHelper::getCategoryInheritedForm($category_id);
								
							if ( empty($form) )
								return;
								
							$data['category_id'] = $category_id;
							$data['form_id'] = $form->id;

							if ($object->show_map && !empty($object->lat) && !empty($object->lng)) {
								// Find the core image field.
								$map_field = RSDirectoryHelper::findFormField('map', $form_fields);

								// If we're using a map field, assign it.
								if ($map_field) {
									$field_model = RSDirectoryModel::getInstance( 'Field', 'RSDirectoryModel', array('ignore_request' => true) );
									$field_model->assign2Form($map_field->id, $form->id);

									// assign the values to the entry
									$data[$map_field->form_field_name]['lat'] = $object->lat;
									$data[$map_field->form_field_name]['lng'] = $object->lng;
								}
							}
								
							// Save entry.
							if ( $entry_model->save($data) )
							{
								// Get the id of the newly inserted entry.
								$entry_id = $entry_model->getState( $entry_model->getName() . '.id' );


								// store the temporary ids
								$tmp = array(
									$db->q($object->link_id),
									$db->q($entry_id),
									$db->q($section),
								);

								$tmp_values[] = implode(',', $tmp);

								// add the other dependencies not saved by the model
								$update_data_object = new stdClass();
								$update_data_object->id 			= $entry_id;
								$update_data_object->avg_rating 	= round($object->link_rating);
								$update_data_object->ratings_count  = $object->link_votes;
								$update_data_object->hits 			= $object->link_hits;

								$db->updateObject('#__rsdirectory_entries', $update_data_object, 'id');

								// Process file uploads
								foreach ($mosetstree_file_uploads as $mosets_file_upload){
									if ( empty($fields_tmp[$mosets_file_upload->cf_id]) )
										continue;

									// Get the RSDirectory! form field id.
									$field_id = $fields_tmp[$mosets_file_upload->cf_id];

									$upload_field_id = RSDirectoryHelper::findElements( array('id' => $field_id), $form_fields );

									// Skip the form field if it wasn't found in the RSDirectory! fields list.
									if ( empty($upload_field) )
										continue;

									$file_path = JPATH_ROOT . '/media/com_mtree/attachments/listings/'.$mosets_file_upload->raw_filename ;
									if (file_exists($file_path)) {
										// Destination directory
										$dst_dir = JPATH_ROOT . "/components/com_rsdirectory/files/entries/$entry_id/";

										// Get the extension.
										$ext = strtolower( JFile::getExt($file_path) );

										// Get a hash.
										$hash = RSDirectoryHelper::getHash();

										// Generate a file name.
										$file_name = $hash . ($ext ? ".$ext" : '');

										if ( JFile::copy($file_path, "$dst_dir/$file_name") )
										{
											// Insert the file.
											$new_file = (object)array(
												'user_id' => $object->user_id,
												'hash' => $hash,
												'file_name' => $file_name,
												'original_file_name' => $mosets_file_upload->filename,
											);

											$db->insertObject('#__rsdirectory_uploaded_files', $new_file, 'id');

											$files[] = array(
												$db->q($new_file->id),
												$db->q($entry_id),
												$db->q($upload_field_id->id),
												$db->q(1),
											);
										}
									}
								}

								// add the images core field type to the form
								$core_image_field = false;
								if (!empty($mosetstree_core_images)) {
									// Find the core image field.
									$core_image_field = RSDirectoryHelper::findFormField('images', $form_fields);

									// If we're using a core Images field, assign it.
									if ($core_image_field) {
										$field_model = RSDirectoryModel::getInstance( 'Field', 'RSDirectoryModel', array('ignore_request' => true) );
										$field_model->assign2Form($core_image_field->id, $form->id);
									}
								}

								// add the map field
								
								// Process image fields.
								$new_order = 1;
								foreach ($mosetstree_image_fields as $mosetstree_image_field)
								{
									// Process the other image fields in the section.
									if ( !$mosetstree_image_field->isCore && empty($fields_tmp[$mosetstree_image_field->cf_id]) )
										continue;

									// Get the RSDirectory! form field id.
									if (!$mosetstree_image_field->isCore) {
										$field_id = $fields_tmp[$mosetstree_image_field->cf_id];

										$image_field = RSDirectoryHelper::findElements(array('id' => $field_id), $form_fields);
									} else {
										if (!$core_image_field) continue;

										$image_field = $core_image_field;
									}
										
									// Skip the form field if it wasn't found in the RSDirectory! fields list.
									if ( empty($image_field) )
										continue;
										
									$properties = $image_field->properties;
										
									$last_width = 0;
									$last_height = 0;
									$last_file_path = '';
										
									foreach (self::$image_sizes as $image_size)
									{
											
										$file_path = JPATH_ROOT . '/media/com_mtree/images/listings/' .$image_size.'/'.$mosetstree_image_field->raw_filename ;
											
										// Skip if the file is not an image
										if ( !RSDirectoryHelper::isImage($file_path) )
											continue;
											
										if ( $properties->get('maximum_file_size') )
										{
											// Skip the file because it's too big.
											if ( $properties->get('maximum_file_size') * 1024 < filesize($file_path) )
												continue;
										}
											
										// Get the extension.
										$ext = strtolower( JFile::getExt($file_path) );
											
										// Get the accepted file extensions.
										$accepted_files = $properties->get('accepted_files') ? explode( "\r\n", $properties->get('accepted_files') ) : array();
											
										if ($accepted_files)
										{
											// Convert all the accepted files extensions to lowercase.
											foreach ($accepted_files as &$value)
											{
												$value = strtolower($value);
											}
												
											// Skip this file if it's unallowed.
											if ( !in_array($ext, $accepted_files) )
												continue;
										}
											
										list($width, $height) = getimagesize($file_path);
											
										if ($width * $height > $last_width * $last_height)
										{
											$last_width = $width;
											$last_height = $height;
											$last_file_path = $file_path;
										}
									}
										
									if ($last_file_path)
									{
										$dst_dir = JPATH_ROOT . "/components/com_rsdirectory/files/entries/$entry_id/";
											
										// Get the extension.
										$ext = strtolower( JFile::getExt($last_file_path) );
											
										// Get a hash.
										$hash = RSDirectoryHelper::getHash();
											
										// Generate a file name.
										$file_name = $hash . ($ext ? ".$ext" : '');
										
										if ( JFile::copy($last_file_path, "$dst_dir/$file_name") )
										{
											// Insert the file.
											$new_file = (object)array(
												'user_id' => $object->user_id,
												'hash' => $hash,
												'file_name' => $file_name,
												'original_file_name' => $mosetstree_image_field->filename,
											);
												
											$db->insertObject('#__rsdirectory_uploaded_files', $new_file, 'id');
												
											$files[] = array(
												$db->q($new_file->id),
												$db->q($entry_id),
												$db->q($image_field->id),
												$db->q($new_order),
											);

											$new_order++;
										}
									}
								}
							}
						}
					}
				}
					
				if ( !empty($files) )
				{
					$columns = array(
						'file_id',
						'entry_id',
						'field_id',
						'ordering',
					);
						
					$query = $db->getQuery(true)
						   ->insert( $db->qn('#__rsdirectory_uploaded_files_fields_relations') )
						   ->columns( $db->qn($columns) );
							
					foreach ($files as $file)
					{
						$query->values( implode(',', $file) );
					}
						
					$db->setQuery($query);
					$db->execute();
				}
			}
		}
		else if ($section == 'reviews') {
			$query = $db->getQuery(true)
				->select('*')
				->from( $db->qn('#__mt_reviews') );

			$db->setQuery($query, $section_offset, $limit);
			$objects = $db->loadObjectList();

			if ($objects) {
				$entries_tmp = $this->getTmp('entries');

				$review_model = RSDirectoryModel::getInstance( 'Rating', 'RSDirectoryModel', array('ignore_request' => true) );

				foreach ($objects as $object) {
					if (!isset($entries_tmp[$object->link_id])) continue;

					$entry_id = $entries_tmp[$object->link_id];

					if (!$user->load($object->user_id)) {
						$object->user_id = 0;
					}

					$query = $db->getQuery(true)
						->select($db->qn('avg_rating'))
						->from($db->qn('#__rsdirectory_entries'))
						->where($db->qn('id') .'='. $db->q($entry_id));

					$db->setQuery($query);
					$avg_rating = (int) $db->loadResult();

					$data = array(
						'entry_id' 		=> $entry_id,
						'user_id'  		=> $object->user_id,
						'ip' 			=> '',
						'score' 		=> $avg_rating,
						'name' 			=> empty($object->user_id) ? $object->guest_name : $user->name,
						'email' 		=> empty($object->user_id) ? '' : $user->email,
						'subject'		=> $object->rev_title,
						'review' 		=> $object->rev_text,
						'owner_reply' 	=> $object->ownersreply_text,
						'created_time' 	=> $object->rev_date,
						'published' 	=> $object->rev_approved

					);

					// save the rating
					$review_model->save($data);
				}
			}
		}
			
		if ($tmp_values)
		{
			$query = $db->getQuery(true)
			       ->insert( $db->qn('#__rsdirectory_import_tmp') )
			       ->columns( $db->qn( array('id_old', 'id_new', 'section') ) )
			       ->values($tmp_values);
					
			$db->setQuery($query);
			$db->execute();
		}
			
		$count = count($objects);
		$totals = $app->getUserState('com_rsdirectory.measure.import.totals');
			
		$vars->response['total'] = $totals['total'];
		
		$progress = $offset + $count;
		$progress = $progress > $totals['total'] ? $totals['total'] : $progress;
			
		$vars->response['progress'] = $progress;
		$vars->response['completition'] = $totals['total'] ? number_format( ($progress * 100) / $totals['total'], 1 ) : 100.0;
			
		$section_progress = $section_offset + $count;
		$section_progress = $section_progress > $totals[$section] ? $totals[$section] : $section_progress;
			
		$vars->response['section_progress'] = $section_progress;
		$vars->response['section_completition'] = $totals[$section] ? number_format( ($section_progress * 100) / $totals[$section], 1 ) : 100.0;
			
		$messages[] = array(
			'message' => JText::sprintf('COM_RSDIRECTORY_IMPORT_SECTION_PROGRESS', $section, $vars->response['section_completition']),
			'type' => 'success',
			'id' => self::sectionNameToId($section),
		);
			
		if ($vars->response['section_completition'] == 100)
		{
			if ($section == 'fields') {
				JFactory::getApplication()->setUserState('com_rsdirectory.measure.import.ismapadded', 0);
			}

			$key = array_search($section, self::$sections);
				
			if ( isset(self::$sections[$key + 1]) )
			{
				// Get the next section.
				$section = self::$sections[$key + 1];
					
				// Reset section completition.
				$vars->response['section_completition'] = 0.0;
					
				// Reset section progress.
				$vars->response['section_progress'] = 0.0;
					
				$messages[] = array(
					'message' => JText::sprintf('COM_RSDIRECTORY_IMPORT_SECTION_PROGRESS', $section, 0.0),
					'type' => 'success',
					'id' => self::sectionNameToId($section),
				);
			}
		}
			
		$vars->response['section'] = $section;
		$vars->response['action'] = $vars->response['completition'] < 100 ? 'import' : 'done';
		$vars->response['messages'] = $messages;
	}

	protected function correlateFieldType($fieldType){
		// remove the core prefix
		$fieldType = stripos($fieldType, 'core') >= 0 ? str_replace('core', '', $fieldType) : $fieldType;

		$found_type = 'textbox';

		switch($fieldType) {
			case 'country':
			case 'price':
			case 'youtube':
				$found_type = $fieldType;
			break;

			case 'desc':
				$found_type = 'description';
			break;

			case 'texteditor':
				$found_type = 'textarea';
			break;

			case 'date':
			case 'mdate':
				$found_type = 'calendar';
			break;

			case 'year':
			case 'myear':
			case 'monthyear':
				$found_type = 'dropdown_date_picker';
			break;

			case 'website':
			case 'weblink':
			case 'vanityurl':
				$found_type = 'link';
			break;

			case 'mfile':
				$found_type = 'fileupload';
			break;

			case 'image':
				$found_type = 'image_upload';
			break;

			case 'radiobutton':
				$found_type = 'radiogroup';
			break;

			case 'selectlist':
			case 'mselectmultiple':
				$found_type = 'dropdown';
			break;

			case 'mcheckbox':
				$found_type = 'checkboxgroup';
			break;

			case 'map':
				$found_type = 'map';
			break;
		}

		return $found_type;
	}
		
	/**
	 * Get data from the import tmp table.
	 *
	 * @access public
	 *
	 * @param string $section
	 *
	 * @return array
	 */
	public function getTmp($section)
	{
		$db = JFactory::getDbo();
			
		$query = $db->getQuery(true)
		       ->select('*')
			   ->from( $db->qn('#__rsdirectory_import_tmp') )
			   ->where( $db->qn('section') . ' = ' . $db->q($section) );
				
		$db->setQuery($query);
			
		$parents = array();
			
		if ( $results = $db->loadObjectList() )
		{
			foreach ($results as $result)
			{
				$parents[$result->id_old] = $result->id_new;
			}
		}
			
		return $parents;
	}
		
	/**
	 * Get the new ids of the specified objects.
	 *
	 * @acces public
	 *
	 * @static
	 *
	 * @param array $parents
	 *
	 * @return array
	 */
	public static function getNewIds($parents, $old_ids)
	{
		$old_ids = (array)$old_ids;
			
		$new_ids = array();
			
		if ($parents && $old_ids)
		{
			foreach ($old_ids as $old_id)
			{
				if ( isset($parents[$old_id]) )
				{
					$new_ids[] = $parents[$old_id];
				}
			}
		}
			
		return $new_ids;
	}
		
	/**
	 * Convert the section name into a valid id attribute string.
	 *
	 * @access public
	 *
	 * @param string $section
	 *
	 * @return string
	 */
	public static function sectionNameToId($section)
	{
		return "import-msg-$section";
	}
		
	/**
	 * A method to generate alias and title.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $title
	 * @param int $parent_id
	 *
	 * @return array An array consisting of the title and alias
	 */
	public static function getTitleAndAlias($title, $parent_id)
	{
		$db = JFactory::getDbo();
		$title = stripslashes($title);
		$alias = JFilterOutput::stringURLSafe(trim($title));
			
		if ( trim( str_replace('-', '', $alias) ) == '' )
		{
			$alias = JFactory::getDate()->format('Y-m-d-H-i-s');
		}
			
		do
		{
			$query = $db->getQuery(true)
			       ->select('COUNT(*)')
				   ->from( $db->qn('#__categories') )
				   ->where( $db->qn('parent_id') . ' = ' . $db->q($parent_id) )
				   ->where( $db->qn('alias') . ' = ' . $db->q($alias) );
					
			$db->setQuery($query);
				
			if ( $result = $db->loadResult() )
			{
				$title = JString::increment($title);
				$alias = JString::increment($alias, 'dash');
			}
		}
		while ($result);
			
		return array($title, $alias);
	}
		
	/**
	 * Update a field property if it already exists in the data array.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param string $prop
	 * @param mixed $value
	 * @param array &$data
	 */
	public static function updateFieldProperty($prop, $value, &$data)
	{
		if ( array_key_exists($prop, $data) )
		{
			$data[$prop] = $value;
		}
	}
		
	/**
	 * Method to check if a MosetsTree category has children.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $category_id
	 *
	 * @return bool
	 */
	public static function hasChildren($category_id)
	{
		static $parents = array();
			
		if (!$parents)
		{
			$db = JFactory::getDbo();
				
			$query = $db->getQuery(true)
				   ->select( $db->qn( array('cat_id', 'cat_parent') ) )
				   ->from( $db->qn('#__mt_cats') );
					
			$db->setQuery($query);
				
			if ( $items = $db->loadObjectList() )
			{
				foreach ($items as $item)
				{
					$parents[$item->cat_parent] = true;
				}
			}
		}
			
		return !empty($parents[$category_id]);
	}
		
	/**
	 * Method to get the lowest subcategories for a specified category.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param int $category_id
	 *
	 * @return array
	 */
	public static function getLowestSubcategories($category_id, $results = array() )
	{
		static $parents = array();
			
		if (!$parents)
		{
			$db = JFactory::getDbo();
				
			$query = $db->getQuery(true)
				   ->select( $db->qn( array('cat_id', 'cat_parent') ) )
				   ->from( $db->qn('#__mt_cats') );
					
			$db->setQuery($query);
				
			if ( $items = $db->loadObjectList() )
			{
				foreach ($items as $item)
				{
					if ( !isset($parents[$item->cat_parent]) )
					{
						$parents[$item->cat_parent] = array();
					}
						
					$parents[$item->cat_parent][] = $item->cat_id;
				}
			}
		}
			
		if ( empty($parents[$category_id]) )
		{
			$results[] = $category_id;
		}
		else
		{
			foreach ($parents[$category_id] as $subcategory_id)
			{
				if ( $results_aux = self::getLowestSubcategories($subcategory_id, $results) )
				{
					$results = array_merge($results, $results_aux);
				}
			}
		}
			
		return array_unique($results);
	}
		
	/**
	 * Get tmp ids.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @param mixed $ids
	 * @param array $tmp_ids
	 *
	 * @return mixed
	 */
	public static function getTmpIds( $ids, $tmp_ids = array() )
	{
		if ( is_array($ids) )
		{
			$results = array();
				
			if ($ids)
			{
				foreach ($ids as $id)
				{
					if ( isset($tmp_ids[$id]) )
					{
						$results[] = $tmp_ids[$id];
					}
				}
			}
				
			return $results;
		}
			
		return isset($tmp_ids[$ids]) ? $tmp_ids[$ids] : null;
	}
		
	/**
	 * Method to check if there are entries stored in parent categories.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @return bool
	 */

	public static function hasEntriesStoredInParentCategories()
	{
		$db = JFactory::getDbo();
			
		$select = array(
			'COUNT('.$db->qn('mt.link_id').')'
		);
			
		$query = $db->getQuery(true)
		       ->select($select)
			   ->from( $db->qn('#__mt_cl', 'mt') )
			   ->where( $db->qn('mt.cat_id') . ' NOT IN (SELECT DISTINCT('.$db->qn('cat_parent').') FROM '.$db->qn('#__mt_cats').')');

		$db->setQuery($query);


		$nr_entries = (int) $db->loadResult();

		return $nr_entries ? true : false;
	}
		
	/**
	 * Method to check if there are entries stored in multiple categories.
	 *
	 * @access public
	 *
	 * @static
	 *
	 * @return bool
	 */

	public static function hasEntriesStoredInMultipleCategories()
	{
		$db = JFactory::getDbo();
			
		$query = $db->getQuery(true)
		       ->select( 'COUNT(*) AS ' . $db->qn('count') )
			   ->from( $db->qn('#__mt_cl') )
			   ->group( $db->qn('link_id') )
			   ->having( $db->qn('count') . ' > ' . $db->q(1) );
				
		$db->setQuery($query);
			
		return (bool)$db->loadResult();
	}
}