<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2014-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

if (!file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php'))
    return;

require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
require_once dirname(__FILE__) . '/helper.php';

$location			= $params->get('location');
$radius				= $params->get('radius');
$default_unit		= $params->get('default_unit');
$circle_color		= $params->get('circle_color', '#ff8080');
$use_geolocation	= $params->get('use_geolocation');
$display_results	= $params->get('display_results');
$show_price_range	= $params->get('show_price_range');
$show_ratings		= $params->get('show_ratings');
$selected_category  = (int) $params->get('selected_category', 0);

// Get the JInput object.
$jinput = JFactory::getApplication()->input;

// Load language.
$lang = JFactory::getLanguage();
$lang->load('com_rsdirectory');

$doc = JFactory::getDocument();

RSDirectoryHelper::loadMedia();

if (!RSDirectoryHelper::isJ4())
{
	JHtml::_('stylesheet', 'mod_rsdirectory_map_search/style.css', array('relative' => true, 'version' => 'auto'));
}

$categories_list = false;
if ($params->get('show_categories')) {
	JHtml::_('script', 'mod_rsdirectory_map_search/script.js', array('relative' => true, 'version' => 'auto'));
        
    // Get categories.
    $categories_list = RSDirectoryHelper::getSubcategories(0);
}

if ($show_ratings) {
	JText::script('COM_RSDIRECTORY_RATY_BAD');
	JText::script('COM_RSDIRECTORY_RATY_POOR');
	JText::script('COM_RSDIRECTORY_RATY_REGULAR');
	JText::script('COM_RSDIRECTORY_RATY_GOOD');
	JText::script('COM_RSDIRECTORY_RATY_GORGEOUS');
	
	RSDirectoryHelper::loadRaty();
}
$use_show_more = (int) $params->get('use_show_more', 0);
if ($use_show_more) {
	JText::script('COM_RSDIRECTORY_SHOW_MORE');
}
$config = JFactory::getConfig();
$limit = (int) $config->get('list_limit');

$directory_config 	= RSDirectoryConfig::getInstance();
$map_type = $directory_config->get('map_type', 'google');

// The opacity is used in both map types
$opacity = (int) $params->get('circle_opacity', 35);

// Constrain the opacity between 0 and 100
$opacity = $opacity > 100 ? 100 : $opacity;
$opacity = $opacity < 0 ? 0 : $opacity;
$opacity = round(($opacity / 100), 2);

if ($map_type == 'google') {
	JHtml::_('script', 'https://maps.google.com/maps/api/js?key='.urlencode($directory_config->get('google_api_key', '')).'&amp;libraries=geometry');
	JHtml::_('script', 'mod_rsdirectory_map_search/jquery.map-search.js', array('relative' => true, 'version' => 'auto'));

	$script = 'jQuery(function($) {
		jQuery(document.getElementById("rsdir-map-search-canvas")).RSMapSearch({
			address: ' . json_encode($location) . ',
			form: document.getElementById("rsdir-map-search"),
			infoWindowWidth: ' . (int)$params->get('info_window_width', 300) . ',
			infoWindowHeight: ' . (int)$params->get('info_window_height', 120) .  ',
			fillColor: ' . json_encode($circle_color) . ',
			fillOpacity: ' . $opacity. ',
            strokeColor: ' . json_encode($params->get('circle_stroke_color', '#000000')) . ',
            hideCircle: ' . (int) $params->get('hide_radius_circle', '0'). ',
			useShowMore: ' . $use_show_more . ',
			limit: '.($limit - 1).'
		});
	});';
} else {
	// leaflet stylesheet
	JHtml::_('stylesheet', 'com_rsdirectory/leaflet.css', array('relative' => true, 'version'=> 'auto'));
	// leafelt
	JHtml::_('script', 'https://unpkg.com/leaflet@1.4.0/dist/leaflet.js', array(), array('integrity' => 'sha512-QVftwZFqvtRNi0ZyCtsznlKSWOStnDORoefr1enyq5mVL4tmKB3S/EnC3rRJcxCPavG10IcrVGSmPh6Qw5lwrg==', 'crossorigin' => ''));

	// esri
	JHtml::_('script', 'https://unpkg.com/esri-leaflet@2.2.4/dist/esri-leaflet.js', array(), array('integrity' => 'sha512-tyPum7h2h36X52O2gz+Pe8z/3l+Y9S1yEUscbVs5r5aEY5dFmP1WWRY/WLLElnFHa+k1JBQZSCDGwEAnm2IxAQ==', 'crossorigin' => ''));

	JHtml::_('script', 'https://unpkg.com/esri-leaflet-geocoder@2.2.13/dist/esri-leaflet-geocoder.js', array(), array('integrity' => 'sha512-zdT4Pc2tIrc6uoYly2Wp8jh6EPEWaveqqD3sT0lf5yei19BC1WulGuh5CesB0ldBKZieKGD7Qyf/G0jdSe016A==', 'crossorigin' => ''));

	JHtml::_('script', 'mod_rsdirectory_map_search/jquery.map-search.os.js', array('relative' => true, 'version'=> 'auto'));

	// specific attributes
	$attr = RSDirectoryHelper::getMapAttribution($map_type, $directory_config);
	$tileType    = $attr['tileType'];
	$accessToken = $attr['accessToken'];
	$attribution = $attr['attribution'];
	$tileLayer   = $attr['tileLayer'];

	$script = 'jQuery(function($)
	{
		jQuery( document.getElementById("rsdir-map-search-canvas") ).RSMapSearchOS(
		{
			address: ' . json_encode($location) . ',
			form: document.getElementById("rsdir-map-search"),
			infoWindowWidth: ' . (int)$params->get('info_window_width', 300) . ',
			infoWindowHeight: ' . (int)$params->get('info_window_height', 120) .  ',
			fillColor: ' . json_encode($circle_color) . ',
			color: ' . json_encode($params->get('circle_stroke_color', '#000000')) . ',
            hideCircle: ' . (int) $params->get('hide_radius_circle'). ',
            fillOpacity: ' . $opacity. ',
			useShowMore: ' . $use_show_more . ',
			limit: '.($limit - 1).',

			tileLayer: \''.$tileLayer.'\',
			tileType: \''.$tileType.'\',
			attribution: \''.$attribution.'\',
			accessToken: \''.$accessToken.'\'
		});

	});';
}

	
// Add the script declaration.
$doc->addScriptDeclaration($script);

JText::script('COM_RSDIRECTORY_LOADING');

require JModuleHelper::getLayoutPath('mod_rsdirectory_map_search', $params->get('layout', 'default'));