<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

/**
 * RSDirectory! Search plugin
 *
 * @package Joomla.Plugin
 * @subpackage RSDirectory!.Entries
 * @since 1.6
 */
class plgSearchRSDirectory extends JPlugin
{
	protected $autoloadLanguage = true;
	/**
	 * @access public
	 *
	 * @return array An array of search areas
	 */
	public function onContentSearchAreas()
	{
		static $areas = array('rsdirectory' => 'PLG_SEARCH_RSDIRECTORY');
		return $areas;
	}
		
	/**
	 * RSEvents!Pro Search method
	 * The sql must return the following fields that are used in a common display
	 * routine: href, title, section, created, text, browsernav
	 *
	 * @access public
	 * 
	 * @param string $text Target search string
	 * @param string $phrase Mathcing option: exact|any|all
	 * @param string $ordering Ordering option: newest|oldest|popular|alpha|category
	 * @param mixed $areas An array if the search it to be restricted to areas, null if search all
	 *
	 * @return array
	 */
	public function onContentSearch($text, $phrase = '', $ordering = '', $areas = null)
	{
		if ( !file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php') ) 
			return array();
			
		// Load language.
		JFactory::getLanguage()->load('plg_search_rsdirectory', JPATH_ADMINISTRATOR);
			
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
		require_once JPATH_ADMINISTRATOR . '/components/com_search/helpers/search.php';
			
		$searchText = $text;
			
		// Check if RSDirectory! has been selected.
		if ( is_array($areas) )
		{
			if ( !array_intersect( $areas, array_keys( $this->onContentSearchAreas() ) ) )
			{
				return array();
			}
		}
			
		$text = trim($text);
			
		if ($text == '')
		{
			return array();
		}
			
		$app = JFactory::getApplication();
		$db	= JFactory::getDbo();
		$user = JFactory::getUser();
		$groups = implode( ',', $user->getAuthorisedViewLevels() );
		$tag = JFactory::getLanguage()->getTag();
			
		$limit = $this->params->def('search_limit', 50);
			
		$wheres = array();
			
		switch ($phrase)
		{
			case 'exact':
					
				$text = $db->q( '%' . $db->escape($text, true) . '%', false );
					
				$wheres2 = array(
					$db->qn('e.title') . " LIKE $text",
					$db->qn('e.description') . " LIKE $text",
				);
					
				$where = '(' . implode(') OR (', $wheres2) . ')';
					
				break;
				
			case 'all':
			case 'any':
			default:
					
				$words = explode(' ', $text);
				$wheres = array();
					
				foreach ($words as $word)
				{
					$word = $db->q( '%' . $db->escape($word, true) . '%', false );
						
					$wheres2 = array(
						$db->qn('e.title') . " LIKE $word",
						$db->qn('e.description') . " LIKE $word",
					);
						
					$wheres[] = implode(' OR ', $wheres2);
				}
					
				$where = '(' . implode( ($phrase == 'all' ? ') AND (' : ') OR ('), $wheres ) . ')';
					
				break;
		}
			
		switch ($ordering)
		{
			case 'oldest':
					
				$order = $db->qn('e.published_time') . ' ASC';
				break;
					
			case 'alpha':
					
				$order = $db->qn('e.title') . ' ASC';
				break;
				
			case 'popular':
					
				$order = $db->qn('e.hits') . ' DESC';
				break;
					
			case 'category':
					
				$order = $db->qn('c.title') . ' ASC, ' . $db->qn('e.title');
				break;
					
			case 'newest':
			default:
					
				$order = $db->qn('e.published_time') . ' DESC';
				break;
		}
			
		$rows = array();
			
		$select = array(
			$db->qn('e.id'),
			$db->qn('e.title'),
			$db->qn('e.description', 'text'),
			$db->qn('e.published_time', 'created'),
			$db->qn('c.title', 'section'),
			$db->q('2') . ' AS ' . $db->qn('browsernav'),
		);
			
		$query = $db->getQuery(true)
		       ->select($select)
		       ->from( $db->qn('#__rsdirectory_entries', 'e') )
			   ->innerJoin( $db->qn('#__categories', 'c') . ' ON ' . $db->qn('e.category_id') . ' = ' . $db->qn('c.id') )
			   ->innerJoin( $db->qn('#__users', 'u') . ' ON ' . $db->qn('e.user_id') . ' = ' . $db->qn('u.id') )
			   ->where('(' . $where . ')')
			   ->where( $db->qn('e.published') . ' = ' . $db->q(1) )
			   ->where( $db->qn('u.block') . ' = ' . $db->q(0) )
			   ->where( '(' . $db->qn('e.expiry_time') . ' = ' . $db->q('0000-00-00 00:00:00') . ' OR ' . $db->qn('e.expiry_time') . ' > ' . $db->q( JFactory::getDate()->toSql() ) . ')' )
			   ->group( $db->qn('e.id') . ', ' . $db->qn('e.title') )
			   ->order($order);
			   
		// Filter by language.
		if ( JLanguageMultilang::isEnabled() )
		{
			$query->where( $db->qn('c.language') . ' IN (' . $db->q($tag) . ',' . $db->q('*') . ')' );
		}
			
		$db->setQuery($query, 0, $limit);
		$entries = $db->loadObjectList();
			
		if ($entries)
		{
			foreach($entries as $i => $entry)
			{
				$entries[$i]->href = RSDirectoryRoute::getEntryURL($entry->id, $entry->title, '', 0, true);
				$entries[$i]->text = strip_tags($entry->text);
				$entries[$i]->section = JText::sprintf('PLG_SEARCH_RSDIRECTORY_PREPEND', $entry->section);
			}
		}
			
		$rows[] = $entries;
			
		$results = array();
			
		if ( count($rows) )
		{
			foreach ($rows as $row)
			{
				$new_row = array();
					
				foreach ($row as $entry)
				{
					if ( searchHelper::checkNoHTML( $entry, $searchText, array('text', 'title') ) )
					{
						$new_row[] = $entry;
					}
				}
					
				$results = array_merge( $results, (array)$new_row );
			}
		}
			
		return $results;
	}
}