<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

// Load the RSDirectory! Plugin helper class file if it exists or stop the execution of the script if it doesn't exist.
if ( file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php') )
{
    require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php';
}
else
{
    return;
}

/**
 * RSDirectory! 2Checkout Payment Plugin.
 */
class plgSystemRSDirectory2Co extends RSDirectoryPlugin
{
    const PAYMENT_METHOD = 'twoco';
    const PLUGIN = 'rsdirectory2co';
    const EXTENSION = 'plg_system_rsdirectory2co';
    const OPTION = 'PLG_SYSTEM_RSDIRECTORY2CO_NAME';
        
    /**
     * Class constructor.
     * 
     * @access public
     * 
     * @param object &$subject
     * @param array $config
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
			
		// Get mainframe.
        $app = JFactory::getApplication();
            
        if ( $app->isClient('administrator') )
        {
            $lang = JFactory::getLanguage();
            $lang->load('com_rsdirectory', JPATH_ADMINISTRATOR);
            $lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
				
			// Show a warning in the admin section if the 2Checkout vendor account number is empty.
            if ( !trim( $this->params->get('id', '') ) )
            {
                $app->enqueueMessage( JText::_('PLG_SYSTEM_RSDIRECTORY2CO_EMPTY_VECTOR_ACCOUNT_NUMBER_ERROR'), 'warning' );
            }
		}
    }
        
    /**
     * Can the plugin run?
     *
     * @access public
     * 
     * @return bool
     */
    public function canRun()
    {
        return parent::canRun() && trim( $this->params->get('id') ) && JPluginHelper::isEnabled('system', self::PLUGIN);
    }
        
    /**
     * On after render.
     *
     * @access public
     */
    public function onAfterRender()
    {
		if ( !$this->canRun() )
			return;
			
		// Get the JInput object.
		$jinput = JFactory::getApplication()->input;
			
		$is_two_old = $jinput->getInt('rsdirectory2coold');
		$is_two_new = $jinput->getInt('rsdirectory2co');
			
		if ($is_two_old || $is_two_new)
		{
			$this->processForm();
		}
    }
        
    /**
     * Add the current payment option to the payments list.
     *
     * @access public
     * 
     * @return string
     */
    public function onRsdirectoryaddPaymentOptions()
    {
		$lang = JFactory::getLanguage();
		$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
			
		if ( !$this->canRun() )
			return;
			
		$tax_text = '';
			
		if ( $tax_value = $this->params->get('tax_value') )
		{
			if ( $this->params->get('tax_type') )
			{
				$tax_text = "$tax_value%";
			}
			else
			{
				$tax_text = RSDirectoryHelper::formatPrice($tax_value);
			}
		}
			
		return (object)array(
			'value' => self::PAYMENT_METHOD,
			'text' => JText::_(self::OPTION),
			'tax_text' => $tax_text,
		);
    }
        
    /**
     * Show payment form.
     *
     * @access public
     * 
     * @param object $vars
     */
    function onRsdirectoryshowForm($vars)
    {
		// Do a few checks and exit the function if the conditions are not met.
		if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
			return;
			
		$lang = JFactory::getLanguage();
		$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
		$lang->load('com_rsdirectory', JPATH_SITE);
			
		$db = JFactory::getDBO();
			
		self::outputTransactionDetails($vars);
			
		?>
			
		<p style="margin: 10px; font-weight: bold;"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORY2CO_REDIRECTING'); ?></p>
			
		<form id="twocoForm" method="post" action="https://www.2checkout.com/checkout/purchase">
			<?php echo !$this->params->get('mode') ? '<input type="hidden" name="demo" value="Y" />' : ''; ?>
			<input type="hidden" name="fixed" value="Y" />
			<input type="hidden" name="sid" value="<?php echo self::escape( $this->params->get('id') ); ?>" />
			<input type="hidden" name="cart_order_id" value="<?php echo self::escape($vars->item_name); ?>" />
			<input type="hidden" name="total" value="<?php echo self::formatPrice($vars->total); ?>" />
			<input type="hidden" name="x_receipt_link_url" value="<?php echo self::escape( JRoute::_( JURI::root() . 'index.php?rsdirectory2coold=1' ) ); ?>" />                  
			<input type="hidden" name="vendor_order_id" value="<?php echo $vars->hash; ?>" />
			<input type="hidden" name="custom" value="<?php echo $vars->hash; ?>" />
			<input type="hidden" name="return_url" value="<?php echo self::escape( JRoute::_( JURI::root() . 'index.php?rsdirectory2coold=1' ) ); ?>" />
		</form>
			
		<script type="text/javascript">
			function twocoFormSubmit()
			{
			   document.getElementById('twocoForm').submit()
			}
				
			try
			{
			   window.addEventListener ? window.addEventListener('load', twocoFormSubmit, false) : window.attachEvent('onload', twocoFormSubmit);
			}
			catch (err)
			{
			   twocoFormSubmit();
			}
		</script>
			
		<?php
    }
        
    /**
     * Process the form.
     *
     * @access private
     */
    private function processForm()
    {
		if ( !$this->canRun() )
			return;
			
			
		$lang = JFactory::getLanguage();
		$lang->load('plg_system_rsdirectory2co', JPATH_ADMINISTRATOR);
			
		// Get DBO.
		$db = JFactory::getDBO();
			
		$filter = JFilterInput::getInstance( array(), array(), 1, 1, 0 );
		$jinput  = new JInput( null, array('filter' => $filter) );
			
		$is_two_old = $jinput->getInt('rsdirectory2coold');
		$is_two_new = $jinput->getInt('rsdirectory2co');
		$custom = $jinput->get('custom', '', 'none');
		$vendor_order_id = $jinput->get('vendor_order_id', '', 'none');
			
			
		if ($is_two_old && $custom)
		{
			$hash = $custom;
		}
			
		if ($is_two_new && $vendor_order_id)
		{
			$hash = $vendor_order_id;
		}
			
		// Exit the function if the hash is empty.
		if ( empty($hash) )
			return;
			
			
		// Get the user transaction.
		$user_transaction = RSDirectoryHelper::getUserTransaction($hash);
			
		// Exit the function if no transaction was found or if the transaction was already finalized.
		if (!$user_transaction || $user_transaction->status == 'finalized')
			return;

		//set the custom fields for the credit purchased email
		$email_custom_fields = array(
			'price-credits' 		=> $user_transaction->price,
			'tax' 					=> $user_transaction->tax,
			'tax-type' 				=> $user_transaction->tax_type,
			'tax-value' 			=> $user_transaction->tax_value,
			'total' 				=> $user_transaction->total,
			'credit-title' 			=> $user_transaction->credit_title,
			'currency' 				=> $user_transaction->currency,
			'credits' 				=> $user_transaction->credits,
			'payment-method' 		=> $user_transaction->gateway,
			'date-purchased' 		=> $user_transaction->date_created,
		);
			
		// Initialize the log array.
		$log = array();
			
		$log[] = "Receiving a new transaction from 2Checkout.";
			
		$two_id = $this->params->get('id');
		$two_secret = $this->params->get('secret_word');
		$order_number = $this->params->get('mode') ? $jinput->get('order_number') : 1;
		$total = $jinput->get('total', '', 'none');
			
		// Require the RSDirectory! helper.
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
			
			
		if ($is_two_old)
		{
			$two_hash = strtoupper( hash('md5', $two_secret . $two_id . $order_number . $total) );
			$two_key = $jinput->get('key', '', 'none');
		}

		$transaction_nok = false;
			
		if ($is_two_new)
		{
			$sale_id = $jinput->get('sale_id', '', 'none');
			$vendor_id = $jinput->get('vendor_id', '', 'none');
			$invoice_id = $jinput->get('invoice_id', '', 'none');
			$two_hash = strtoupper( hash('md5', $sale_id . $vendor_id . $invoice_id . $two_secret) );
			$two_key = $jinput->get('md5_hash', '', 'none');
		}
			
		if ( empty($two_hash) || empty($two_key) ) 
		{
			$log[] = 'Transaction did not pass verification; empty Hash or Key.';
			RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
			$transaction_nok = true;
		}
			
		if ($two_hash != $two_key && !$transaction_nok)
		{
			$log[] = "Transaction did not pass verification; expected hash: $two_key, computed: $two_hash.";
			RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
			$transaction_nok = true;
		}
			
			
		if ( $is_two_old && self::formatPrice($user_transaction->total) != self::formatPrice($total) && !$transaction_nok)
		{
			$log[] = "Expected an amount of $user_transaction->total $user_transaction->currency. 2Checkout reports this payment is $total $user_transaction->currency. Stopping.";
			RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
			$transaction_nok = true;
		}

		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';

		// send the refusal email if the transaction is not ok and return
		if ($transaction_nok) {
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the form associated to the category.
				$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

				// Get the published form fields assigned to the form.
				$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
			} else {
				$entry = (object)array('skip_entry' => true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_no_entry', null, $entry, null, $email_custom_fields)->send();
			}

			return;
		}
			
		// Process payment.
		$processed = $jinput->get('credit_card_processed');

			
		if ($is_two_new)
		{
			$processed = 'Y';
		}
			
		if ($processed == 'Y')
		{
			// Set the transaction params.
			$params = array();
				
			if ( !empty($_POST) )
			{
				foreach ($_POST as $key => $value)
				{
					$params[] = "$key=$value";
				}
			}
				
			if ( !empty($_GET) )
			{
				foreach ($_GET as $key => $value)
				{
					$params[] = "$key=$value";
				}
			}
				
			$new_user_transaction = (object)array(
				'id' => $user_transaction->id,
				'gateway_order_number' => $order_number,
				'gateway_params' => implode("\n", $params),
				'date_finalized' => JFactory::getDate()->toSql(),
				'finalized' => 1,
				'status' => 'finalized',
			);
				
			JFactory::getDbo()->updateObject('#__rsdirectory_users_transactions', $new_user_transaction, 'id');
				
			RSDirectoryCredits::addUserCredits($user_transaction->user_id, $user_transaction->credits);
				
			$log[] = "Successfully added payment to database.";

			// set the email custom fields
			$email_custom_fields['gateway-order-number'] = $new_user_transaction->gateway_order_number;
			$email_custom_fields['gateway-order-type'] 	 = '';
			$email_custom_fields['date-finalized'] 		 = $new_user_transaction->date_finalized;
			$email_custom_fields['status'] 				 = JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($new_user_transaction->status));
			
			// Get entry.
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the number of unpaid entry credits.
				$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);

				// Get the total number of credits available to the user
				$user_credits = RSDirectoryCredits::getUserCredits($user_transaction->user_id);

				// Get plugin entry publishing param.
				$publish_entry = (int) $this->params->get('publish_entry', 0);

				// Verify if this entry can be published automatically.
				$automatically_publish_entry = $publish_entry == 1 || ($publish_entry == 2 && RSDirectoryHelper::checkUserPermission('auto_publish_entries', $user_transaction->user_id));

				// Check if enough credits are available.
				$has_enough_credits = $user_credits == 'unlimited' || $cost <= $user_credits;

				if ($automatically_publish_entry && $has_enough_credits) {
					// Holds emails that will be sent later on.
					$emails = array('credit_purchase_processed_finalized_with_entry');

					// Setup new entry data that should be overridden.
					$entry_data = (object)array(
							'id' => $entry->id,
							'paid' => 1,
							'published' => 1,
							'published_time' => JFactory::getDate()->toSql()
					);

					// Calculate expiry time.
					if ($entry->period) {
						$expiry_time = JFactory::getDate($entry_data->published_time);
						$expiry_time->modify("+{$entry->period} days");

						$entry_data->expiry_time = $expiry_time->toSql();
					} else {
						$entry_data->expiry_time = $db->getNullDate();
					}

					// Flag to send emails if the entry wasn't published and now it is.
					if (!$entry->published) {
						$emails[] = 'publish_entry';
					}

					// Subtract number of credits if the user doesn't have unlimited credits
					if ($user_credits != 'unlimited') {
						// Charge user.
						$query = $db->getQuery(true)
								->update($db->qn('#__rsdirectory_users'))
								->set($db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost))
								->where($db->qn('user_id') . ' = ' . $db->q($user_transaction->user_id));

						$db->setQuery($query);
						$db->execute();
					}

					// Mark entry credits as being paid.
					$query = $db->getQuery(true)
							->update($db->qn('#__rsdirectory_entries_credits'))
							->set($db->qn('paid') . ' = ' . $db->q($entry_data->paid))
							->where($db->qn('entry_id') . ' = ' . $db->q($entry_data->id));

					$db->setQuery($query);
					$db->execute();

					// Save entry data.
					$db->updateObject('#__rsdirectory_entries', $entry_data, 'id');

					$log[] = 'Automatically published entry #'.$entry->id.'.';

					// Send emails.
					if ($emails) {
						// Get the form associated to the category.
						$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

						// Get the published form fields assigned to the form.
						$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

						// Send emails
						foreach ($emails as $email) {
							RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form, ($email == 'credit_purchase_processed_finalized_with_entry' ? $email_custom_fields : null))->send();
							$log[] = "Sent $email.";
						}
					}
				} else {
					// Get the form associated to the category.
					$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

					// Get the published form fields assigned to the form.
					$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

					RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
					$log[] = "Sent credit_purchase_processed_finalized_with_entry.";
				}
			} else {
				$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
				$log[] = "Sent credit_purchase_processed_finalized_no_entry.";
			}

			// save all the log, including the mails sent
			RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
				
			$redirect_url = $this->params->get('redirect_url');
				
			// The redirect_url parameter was not set.
			if (!$redirect_url)
			{
				$redirect_url = JRoute::_('index.php?option=com_rsdirectory&view=myaccount&tab=transactions', false);
			}
				
			JFactory::getApplication()->redirect($redirect_url);
		}
    }
		
	/**
     * Get taxa data.
     *
     * @access public
     *
     * @param object $vars
     *
     * @return array
     */
    public function onRsdirectorygetTaxData($vars)
    {
        if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
            return;
            
        // Calculate the tax.
        $tax_type = $this->params->get('tax_type');
		$tax_value = $this->params->get('tax_value');
        $tax = $tax_value && $tax_type ? $vars->price * ($tax_value/100) : $tax_value;
            
        $data = array(
            'tax_type' => $tax_type ? 'percent' : 'fixed',
            'tax_value' => $tax_value,
            'tax' => $tax,
        );
			
		return $data;
    }
}