<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/copyleft/gpl.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

// Load the RSDirectoryPlugin helper class file if it exists or stop the execution of the script if it doesn't exist.
if ( file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php') )
{
    require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php';
}
else
{
    return;
}

/**
 * RSDirectory! Authorize Payment Plugin.
 */
class plgSystemRSDirectoryAuthorize extends RSDirectoryPlugin
{
    const PAYMENT_METHOD = 'authorize';
    const PLUGIN = 'rsdirectoryauthorize';
    const EXTENSION = 'plg_system_rsdirectoryauthorize';
    const OPTION = 'PLG_SYSTEM_RSDIRECTORYAUTHORIZE_NAME';
        
    /**
     * Class constructor.
     * 
     * @access public
     * 
     * @param object &$subject
     * @param array $config
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
            
        // Get mainframe.
        $app = JFactory::getApplication();
            
        if ( $app->isClient('administrator') )
        {
            $lang = JFactory::getLanguage();
            $lang->load('com_rsdirectory', JPATH_ADMINISTRATOR);
            $lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
                
            // Show a warning in the admin section if the API Login ID and/or Transaction key are empty.    
            if ( !trim( $this->params->get('id', '') ) || !trim( $this->params->get('key', '') ) )
            {
                $app->enqueueMessage( JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_EMPTY_ERROR'), 'warning' );
            }
        }
    }
        
    /**
     * Can the plugin run?
     *
     * @access public
     * 
     * @return bool
     */
    public function canRun()
    {
        return parent::canRun() && trim( $this->params->get('id', '') ) && trim( $this->params->get('key', '') ) && JPluginHelper::isEnabled('system', self::PLUGIN);
    }
        
    /**
     * Add the current payment option to the payments list.
     *
     * @access public
     * 
     * @return string
     */
    public function onRsdirectoryaddPaymentOptions()
    {
        $lang = JFactory::getLanguage();
        $lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
            
        if ( !$this->canRun() )
			return;
            
        $tax_text = '';
			
        if ( $tax_value = $this->params->get('tax_value') )
		{
			if ( $this->params->get('tax_type') )
			{
				$tax_text = "$tax_value%";
			}
			else
			{
				$tax_text = RSDirectoryHelper::formatPrice($tax_value);
			}
		}
			
		return (object)array(
			'value' => self::PAYMENT_METHOD,
			'text' => JText::_(self::OPTION),
			'tax_text' => $tax_text,
		);
    }
        
    /**
     * Show payment form.
     *
     * @access public
     * 
     * @param object $vars
     */
    public function onRsdirectoryshowForm($vars)
    {
        if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
            return;
            
            
        $lang = JFactory::getLanguage();
        $lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
        $lang->load('com_rsdirectory', JPATH_SITE);
            
        $db = JFactory::getDBO();
        $app = JFactory::getApplication();
            
        $data_aux = array(
            'cc_number' => '',
            'cc_exp_m' => '',
            'cc_exp_y' => '',
            'cc_csc' => '',
            'firstname' => '',
            'lastname' => '',
            'country' => '',
            'city' => '',
            'state' => '',
            'address' => '',
            'zipcode' => '',
        );
            
        // Get data.
        $data = $app->getUserState('com_rsdirectory.authorizenet.data');
        $data = (object)array_merge( $data_aux, empty($data) ? array() : $data );
            
        $error_fields = $app->getUserState('com_rsdirectory.authorizenet.error_fields');
        $error_fields = $error_fields ? $error_fields : array();

        JHtml::_('behavior.keepalive');
            

        JHtml::_('script', 'com_rsdirectory/script.js', array('relative' => true, 'version' => 'auto'));
            
            
        // Create the months dropdown.
        $months = array(
            JHtml::_( 'select.option', '', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_SELECT_MONTH') ),
            JHtml::_( 'select.option', '01', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_JANUARY') ),
            JHtml::_( 'select.option', '02', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_FEBRUARY') ),
            JHtml::_( 'select.option', '03', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_MARCH') ),
            JHtml::_( 'select.option', '04', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_APRIL') ),
            JHtml::_( 'select.option', '05', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_MAY') ),
            JHtml::_( 'select.option', '06', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_JUNE') ),
            JHtml::_( 'select.option', '07', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_JULY') ),
            JHtml::_( 'select.option', '08', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_AUGUST') ),
            JHtml::_( 'select.option', '09', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_SEPTEMBER') ),
            JHtml::_( 'select.option', '10', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_OCTOBER') ),
            JHtml::_( 'select.option', '11', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_NOVEMBER') ),
            JHtml::_( 'select.option', '12', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_DECEMBER') ),
        );
            
        $exp_month = JHtml::_( 'select.genericlist', $months, 'jform[cc_exp_m]', (RSDirectoryHelper::isJ4() ? 'class="form-select"' : ''), 'value', 'text', $data->cc_exp_m );
            
        $current_year = date( 'Y', JFactory::getDate()->toUnix () );
        $limit_year = $current_year + 10;
            
        $years = array(
            JHtml::_( 'select.option', '', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_SELECT_YEAR') ),
        );
            
        for ( $i = $current_year; $i <= $limit_year; $i++)
        {
            $years[] = JHtml::_('select.option', $i, $i);
        }
            
            
        $exp_year = JHtml::_( 'select.genericlist', $years, 'jform[cc_exp_y]', (RSDirectoryHelper::isJ4() ? 'class="form-select"' : ''), 'value', 'text', $data->cc_exp_y );
            
        // Get countries.
        $query = $db->getQuery(true)
               ->select('*')
               ->from( $db->qn('#__rsdirectory_countries') );
           
        $db->setQuery($query);
            
        $countries = $db->loadObjectList();
            
        // Initialize countries options.
        $countries_options = array(
            JHtml::_( 'select.option', '', JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_SELECT_COUNTRY') ),
        );
            
        foreach ($countries as $country)
        {
            $countries_options[] = JHtml::_('select.option', $country->name, $country->name);
        }
            
        $clist = JHtml::_('select.genericlist', $countries_options, 'jform[country]', (RSDirectoryHelper::isJ4() ? 'class="form-select"' : ''), 'value', 'text', $data->country, 'country');
            
        $uri = JUri::getInstance();
            
        if ( $this->params->get('ssl', '') )
        {
            $uri->setScheme('https');
        }
            
        ?>
		
		<style type="text/css">
		#rs_whats_csc {
			border-bottom: 1px dotted #666;
			cursor: pointer;
		}
		</style>
            
        <form class="form-horizontal" id="rsdir-authorize" name="paymentForm" method="post" action="<?php echo htmlspecialchars( $uri->toString() ); ?>" autocomplete="off">
                
            <?php self::outputTransactionDetails($vars); ?>
                
            <div class="control-group<?php echo in_array('cc_number', $error_fields) ? ' error' : ''; ?>">
                <label class="control-label" for="cc_number">
                    <?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_NUMBER'); ?>
                </label>
                <div class="controls">
                    <input id="cc_number" type="text" name="jform[cc_number]"<?php echo RsdirectoryAdapterGrid::formControl(true);?> onkeyup="javascript:this.value=this.value.replace(/[^0-9]/g, '');" maxlength="16" size="25" value="<?php echo $data->cc_number; ?>" />
                    <br /><br />
                    <img src="<?php echo JURI::root(true); ?>/media/com_rsdirectory/images/cc_logo.gif" alt="CC" />
                </div>
            </div>
                
            <div class="control-group<?php echo in_array('cc_exp_m', $error_fields) ? ' error' : ''; ?>">
                <label class="control-label">
                    <?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_EXP_DATE'); ?>
                </label>
                <div class="controls">
                    <?php if (RSDirectoryHelper::isJ4()) { ?>
                    <div class="row">
                        <div class="col-auto"><?php echo $exp_month; ?></div>
                        <div class="col-auto">/</div>
                        <div class="col-auto"><?php echo $exp_year; ?></div>
                    </div>
                    <?php } else {
                        echo "$exp_month / $exp_year";
                    } ?>
                </div>
            </div>
                
            <div class="control-group<?php echo in_array('cc_csc', $error_fields) ? ' error' : ''; ?>">
                <label class="control-label" for="cc_csc">
                    <?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_CCV'); ?>
                </label>
                <div class="controls">
                    <?php if (RSDirectoryHelper::isJ4()) { ?>
                    <div class="row">
                        <div class="col-auto">
                            <div class="input-group mb-3">
                    <?php } ?>
                        <input id="cc_csc"<?php echo RsdirectoryAdapterGrid::formControl(true);?>  type="text" name="jform[cc_csc]" size="5" onkeyup="javascript:this.value=this.value.replace(/[^0-9]/g, '');" maxlength="4" value="<?php echo $data->cc_csc; ?>" />
                        <span class="input-group-text" id="rs_whats_csc" onmouseover="rs_tooltip.show('rs_tooltip');" onmouseout="rs_tooltip.hide();"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_CCV_WHATS_THIS'); ?></span>
                    <?php if (RSDirectoryHelper::isJ4()) { ?>
                            </div>
                        </div>
                    </div>
                    <?php } ?>
                 </div>
            </div>
                
            <div class="control-group<?php echo in_array('firstname', $error_fields) ? ' error' : ''; ?>">
                <label class="control-label" for="firstname"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_FIRST_NAME'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?>  id="firstname" type="text" name="jform[firstname]" size="25" value="<?php echo $data->firstname; ?>" />
                </div>
            </div>
                
            <div class="control-group<?php echo in_array('lastname', $error_fields) ? ' error' : ''; ?>">
                <label class="control-label" for="lastname"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_LAST_NAME'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?>  id="lastname" type="text" name="jform[lastname]" size="25" value="<?php echo $data->lastname; ?>" />
                </div>
            </div>
                
            <div class="control-group">
                <label class="control-label" for="country"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_COUNTRY'); ?></label>
                <div class="controls">
                    <?php echo $clist; ?>
                </div>
            </div>
                
            <div class="control-group">
                <label class="control-label" for="city"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CITY'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?>  id="city" type="text" name="jform[city]" size="25" value="<?php echo $data->city; ?>" />
                </div>
            </div>
                
            <div class="control-group">
                <label class="control-label" for="state"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_STATE'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?>  id="state" type="text" name="jform[state]" size="25" value="<?php echo $data->state; ?>" />
                </div>
            </div>
                
            <div class="control-group">
                <label class="control-label" for="address"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_ADDRESS'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?> id="address" type="text" name="jform[address]" size="25" value="<?php echo $data->address; ?>" />
                </div>
            </div>
                
            <div class="control-group">
                <label class="control-label" for="zipcode"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_ZIPCODE'); ?></label>
                <div class="controls">
                    <input<?php echo RsdirectoryAdapterGrid::formControl(true);?>  id="zipcode" type="text" name="jform[zipcode]" size="6" value="<?php echo $data->zipcode; ?>" />
                </div>
            </div>
                
            <button class="btn btn-primary" type="submit"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_PAY'); ?></button>
                
            <?php echo JHtml::_('form.token') . "\n"; ?>
            <input type="hidden" name="task" value="credits.authorizenet" />
            <input type="hidden" name="jform[user_transaction_id]" value="<?php echo self::escape($vars->user_transaction_id); ?>" />
        </form>
            
        <div id="rs_tooltip" style="display: none;"> 
            <div><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_CARD_CCV_DESC'); ?></div>
            <div align="center"><img src="<?php echo JURI::root(true); ?>/media/com_rsdirectory/images/cc_csc.gif" alt="CSC" /></div>
        </div>
            
        <?php
            
        $app->setUserState('com_rsdirectory.authorizenet.data', null);
        $app->setUserState('com_rsdirectory.authorizenet.error_fields', null);
            
        JText::script('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_VALIDATE_CC_NUMBER');
        JText::script('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_VALIDATE_CC_CCV');
        JText::script('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_VALIDATE_CC_EXP');
        JText::script('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_VALIDATE_FIRST_NAME');
        JText::script('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_VALIDATE_LAST_NAME');
    }
        
    /**
     * Get taxa data.
     *
     * @access public
     *
     * @param object $vars
     *
     * @return array
     */
    public function onRsdirectorygetTaxData($vars)
    {
        if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
            return;
            
        // Calculate the tax.
        $tax_type = $this->params->get('tax_type');
		$tax_value = $this->params->get('tax_value');
        $tax = $tax_value && $tax_type ? $vars->price * ($tax_value/100) : $tax_value;
            
        $data = array(
            'tax_type' => $tax_type ? 'percent' : 'fixed',
            'tax_value' => $tax_value,
            'tax' => $tax,
        );
			
		return $data;
    }
        
    /**
     * Process the form.
     *
     * @access public
     *
     * @param object $vars
     */
    public function onRsdirectoryProcessForm($vars)
    {
        // Get mainframe.
        $app = JFactory::getApplication();
            
        if ( !$this->canRun() )
            return;
            
            
        // Do a few checks and exit the function if something was wrong.
        if ( $app->isClient('administrator') || empty($vars->user_transaction_id) )
            return;
            
        // Get the user transaction.
		$user_transaction = RSDirectoryHelper::getUserTransaction($vars->user_transaction_id);
            
        // Exit the function if no transaction was found or if the transaction was already finalized.
		if (!$user_transaction || $user_transaction->status == 'finalized')
			return;
            
        // Load language files.
        $lang = JFactory::getLanguage();
        $lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
        $lang->load('com_rsdirectory', JPATH_SITE);
            
            
        // Require the RSDirectory! helper.
        require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
            
            
        @ob_end_clean();
            
            
        // Get user transaction object.
        $user_transaction = RSDirectoryHelper::getUserTransaction($vars->user_transaction_id);
            
        // Get user.
        $user = JFactory::getUser($user_transaction->user_id);
            
            
        // Set x_invoice_num.
        $x_invoice_num = uniqid( $this->params->get('x_login') . ' ' . $this->params->get('x_tran_key') );
        $x_invoice_num = substr( hash('md5', $x_invoice_num), 0, 20 );
            
        $post_values = array(
            'x_login' => $this->params->get('id'),
            'x_tran_key' => $this->params->get('key'),
                
            'x_version' => '3.1',
            'x_delim_data' => 'TRUE',
            'x_delim_char' => '|',
            'x_relay_response' => 'FALSE',
                
            'x_type' => 'AUTH_CAPTURE',
            'x_method' => 'CC',
            'x_card_num' => $vars->cc_number,
            'x_exp_date' => "$vars->cc_exp_m-$vars->cc_exp_y",
            'x_card_code' =>$vars->cc_csc,
                
            'x_amount' => $user_transaction->total,
            'x_currency_code' => $user_transaction->currency,
            'x_invoice_num' => $x_invoice_num,
            'x_description' => self::escape($user_transaction->credit_title),
                
            'x_first_name' => $vars->firstname,
            'x_last_name' => $vars->lastname,
            'x_email' => $user->email,
            'x_country' => $vars->country,
            'x_city' => $vars->city,
            'x_state' => $vars->state,
            'x_address' => $vars->address,
            'x_zip' => $vars->zipcode,
        );
            
        $post_url = $this->params->get('mode') ? 'https://secure.authorize.net/gateway/transact.dll' : 'https://test.authorize.net/gateway/transact.dll';

        $postData = array();

        foreach ($post_values as $key => $value)
        {
            $postData[] = $key."=".urlencode($value);
        }

        $postData = implode('&', $postData);
		
		// set the custom fields for the emails
		$email_custom_fields = array(
			'price-credits' 		=> $user_transaction->price,
			'tax' 					=> $user_transaction->tax,
			'tax-type' 				=> $user_transaction->tax_type,
			'tax-value' 			=> $user_transaction->tax_value,
			'total' 				=> $user_transaction->total,
			'credit-title' 			=> $user_transaction->credit_title,
			'currency' 				=> $user_transaction->currency,
			'credits' 				=> $user_transaction->credits,
			'payment-method' 		=> $user_transaction->gateway,
			'date-purchased' 		=> $user_transaction->date_created,
		);

		$send_failed_email = true;
		$redirect_error = array();

		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';

        $http = JHttpFactory::getHttp();

        // hold the connection response
        $response = '';
        $error = '';

        try
        {
            $request = $http->post($post_url, $postData, array(), 5);

            $code = $request->code;

            if ($code != 200)
            {
                throw new Exception(sprintf('Connection Error: %d', $request->code));
            }

            // set the response
            $response = (string) $request->body;

            if (!strlen($response))
            {
                throw new Exception('Response does not contain a valid body');
            }
        }
        catch (Exception $e)
        {
            $error = $e->getMessage();
        }
            
        if (strlen($response))
        {
            // Initialize the log array.
            $log = array();
                
            // This line takes the response and breaks it into an array using the specified delimiting character.
            $response = explode('|', $response);
                
            if ($response[0] == 1 && $x_invoice_num == $response[7])
            {
                // Format the credits package price.
				$total = self::formatPrice($user_transaction->total);
				$payment_amount = $response[9];
				$payment_currency = $user_transaction->currency;
                if ($total == $payment_amount)
                {
                    $new_user_transaction = (object)array(
                        'id' => $vars->user_transaction_id,
                        'gateway_order_number' => $response[6],
                        'date_finalized' => JFactory::getDate()->toSql(),
                        'finalized' => 1,
                        'status' => 'finalized',
                    );

                    $db = JFactory::getDbo();

                    $db->updateObject('#__rsdirectory_users_transactions', $new_user_transaction, 'id');
                        
                    // Add credits.
                    RSDirectoryCredits::addUserCredits($user_transaction->user_id, $user_transaction->credits);
					
					$log[] = 'Start posting one off payment...';
                    $log[] = "POSTed to $post_url:";
                    $log[] = str_replace("&x_card_num=$vars->cc_number", '&x_card_num=<HIDDEN>', $postData);
                    $log[] = 'Response:';
                    $log[] = implode('|', $response);
                    $log[] = 'Transaction ID extracted from Response: ' . $response[6];
                    $log[] = 'End posting one off payment.';
					
					// set the email custom fields
					$email_custom_fields['gateway-order-number'] = $new_user_transaction->gateway_order_number;
					$email_custom_fields['gateway-order-type'] 	 = '';
					$email_custom_fields['date-finalized'] 		 = $new_user_transaction->date_finalized;
					$email_custom_fields['status'] 				 = JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($new_user_transaction->status));


					// Get entry.
					if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
						// Get the number of unpaid entry credits.
						$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);

						// Get the total number of credits available to the user
						$user_credits = RSDirectoryCredits::getUserCredits($user_transaction->user_id);

						// Get plugin entry publishing param.
						$publish_entry = (int) $this->params->get('publish_entry', 0);

						// Verify if this entry can be published automatically.
						$automatically_publish_entry = $publish_entry == 1 || ($publish_entry == 2 && RSDirectoryHelper::checkUserPermission('auto_publish_entries', $user_transaction->user_id));

						// Check if enough credits are available.
						$has_enough_credits = $user_credits == 'unlimited' || $cost <= $user_credits;

						if ($automatically_publish_entry && $has_enough_credits) {
							// Holds emails that will be sent later on.
							$emails = array('credit_purchase_processed_finalized_with_entry');

							// Setup new entry data that should be overridden.
							$entry_data = (object)array(
									'id' => $entry->id,
									'paid' => 1,
									'published' => 1,
									'published_time' => JFactory::getDate()->toSql()
							);

							// Calculate expiry time.
							if ($entry->period) {
								$expiry_time = JFactory::getDate($entry_data->published_time);
								$expiry_time->modify("+{$entry->period} days");

								$entry_data->expiry_time = $expiry_time->toSql();
							} else {
								$entry_data->expiry_time = $db->getNullDate();
							}

							// Flag to send emails if the entry wasn't published and now it is.
							if (!$entry->published) {
								$emails[] = 'publish_entry';
							}

							// Subtract number of credits if the user doesn't have unlimited credits
							if ($user_credits != 'unlimited') {
								// Charge user.
								$query = $db->getQuery(true)
										->update($db->qn('#__rsdirectory_users'))
										->set($db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost))
										->where($db->qn('user_id') . ' = ' . $db->q($user_transaction->user_id));

								$db->setQuery($query);
								$db->execute();
							}

							// Mark entry credits as being paid.
							$query = $db->getQuery(true)
									->update($db->qn('#__rsdirectory_entries_credits'))
									->set($db->qn('paid') . ' = ' . $db->q($entry_data->paid))
									->where($db->qn('entry_id') . ' = ' . $db->q($entry_data->id));

							$db->setQuery($query);
							$db->execute();

							// Save entry data.
							$db->updateObject('#__rsdirectory_entries', $entry_data, 'id');

							$log[] = 'Automatically published entry #'.$entry->id.'.';

							// Send emails.
							if ($emails) {
								// Get the form associated to the category.
								$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

								// Get the published form fields assigned to the form.
								$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

								// Send emails
								foreach ($emails as $email) {
									RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form, ($email == 'credit_purchase_processed_finalized_with_entry' ? $email_custom_fields : null))->send();
									$log[] = "Sent $email.";
								}
							}
						} else {
							// Get the form associated to the category.
							$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

							// Get the published form fields assigned to the form.
							$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

							RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
							$log[] = "Sent credit_purchase_processed_finalized_with_entry.";
						}
						$send_failed_email = false;
					} else {
						$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
						RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
						
						$log[] = "Sent credit_purchase_processed_finalized_no_entry.";
						
						$send_failed_email = false;
					}
         
                    RSDirectoryHelper::saveTransactionLog($vars->user_transaction_id, $log);
                        
                    $redirect_url = $this->params->get('redirect_url');
                        
                    // The redirect_url parameter was not set.
                    if (!$redirect_url)
                    {
                        $redirect_url = JRoute::_('index.php?option=com_rsdirectory&view=myaccount&tab=transactions', false);
                    }
                        
                    // Clear the data from the session.
                    $app->setUserState('com_rsdirectory.authorizenet.data', null);
                        
                    // Clear the transaction id from the session.
                    $app->setUserState('com_rsdirectory.credits.id', null);

                    $app->enqueueMessage(JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_TRANSACTION_FINALIZED'));
                    
                    $app->redirect( $redirect_url);
                }
                else
                {
                    $amount_error = "Expected an amount of $total $user_transaction->currency. Authorize.NET reports this payment is $payment_amount $payment_currency.";
                        
                    $log[] = "Expected an amount of $total $user_transaction->currency. Authorize.NET reports this payment is $payment_amount $payment_currency. Stopping.";
					
					RSDirectoryHelper::saveTransactionLog($vars->user_transaction_id, $log);
					
					$redirect_error['msg'] = $amount_error;
					$redirect_error['type'] = 'error';
                }
            }
            else
            {
                $log[] = 'Authorize.NET reported the following error: ' . $response[3];
                    
                RSDirectoryHelper::saveTransactionLog($vars->user_transaction_id, $log);
				
				$redirect_error['msg'] = self::escape($response[3]);
				$redirect_error['type'] = $response[0] == 4 ? 'warning' : 'error';
            }
        }
        else
        {
			$redirect_error['msg'] = JText::_('PLG_SYSTEM_RSDIRECTORYAUTHORIZE_GENERAL_ERROR') . " ($error)";
			$redirect_error['type'] = 'error';
        }
		
		if($send_failed_email) {
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the form associated to the category.
				$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

				// Get the published form fields assigned to the form.
				$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
			} else {
				$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_no_entry', null, $entry, null, $email_custom_fields)->send();
			}
		}
        // Enqueue the redirect message
        $app->enqueueMessage($redirect_error['msg'], $redirect_error['type']);
		
		$app->redirect( JUri::getInstance()->toString());
    }
        
    /**
     * Get values between tags.
     *
     * @access private
     * 
     * @static
     * 
     * @param string $haystack
     * @param string $tag
     *
     * @return string
     */
    private static function getTagValue($haystack, $tag) 
    {
        $start = "<$tag>";
        $end = "</$tag>";
            
        if ( strpos($haystack, $start) === false || strpos($haystack, $end) === false ) 
            return false;
            
        $start_position = strpos($haystack, $start) + strlen($start);
        $end_position = strpos($haystack, $end);
            
        return substr($haystack, $start_position, $end_position - $start_position);
    }
}