<?php
/**
 * @package RSDirectory!
 * @copyright (C) 2013-2021 www.rsjoomla.com
 * @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

// Load the RSDirectory! Plugin helper class file if it exists or stop the execution of the script if it doesn't exist.
if ( file_exists(JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php') )
{
    require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/plugin.php';
}
else
{
    return;
}

/**
 * RSDirectory! PayPal Payment Plugin.
 */
class plgSystemRSDirectoryPayPal extends RSDirectoryPlugin
{
    const PAYMENT_METHOD = 'paypal';
    const PLUGIN = 'rsdirectorypaypal';
    const EXTENSION = 'plg_system_rsdirectorypaypal';
    const OPTION = 'PLG_SYSTEM_RSDIRECTORYPAYPAL_NAME';
        
    /**
     * Class constructor.
     * 
     * @access public
     * 
     * @param object &$subject
     * @param array $config
     */
    public function __construct(&$subject, $config)
    {
		parent::__construct($subject, $config);
			
		$app = JFactory::getApplication();
			
		// Show a warning in the admin section if PayPal Merchant email is not defined.
		if ( $app->isClient('administrator') )
		{
			$lang = JFactory::getLanguage();
			$lang->load('com_rsdirectory', JPATH_ADMINISTRATOR);
			$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
				
			if ( !JMailHelper::isEmailAddress( $this->params->get('paypal_email', '') ) )
			{
				$app->enqueueMessage( JText::_( 'PLG_SYSTEM_RSDIRECTORYPAYPAL_NO_VALID_EMAIL'), 'warning' );
			}
		}
    }
        
    /**
     * Can the plugin run?
     *
     * @access public
     * 
     * @return bool
     */
    public function canRun()
    {
        return parent::canRun() && JMailHelper::isEmailAddress( $this->params->get('paypal_email', '') ) && JPluginHelper::isEnabled('system', self::PLUGIN);
    }
        
    /**
     * On after initialise.
     *
     * @access public
     */
    public function onAfterInitialise()
    {	
		if ( $this->canRun() && JFactory::getApplication()->input->get->get('rsdirectorypaypal') )
		{
			$this->processForm();
		}
    }
     
    /**
     * Add the current payment option to the payments list.
     *
     * @access public
     * 
     * @return string
     */
    public function onRsdirectoryaddPaymentOptions()
    {
		$lang = JFactory::getLanguage();
		$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
			
		if ( !$this->canRun() )
			return;
			
		$tax_text = '';
			
        if ( $tax_value = $this->params->get('tax_value') )
		{
			if ( $this->params->get('tax_type') )
			{
				$tax_text = "$tax_value%";
			}
			else
			{
				$tax_text = RSDirectoryHelper::formatPrice($tax_value);
			}
		}
			
		return (object)array(
			'value' => self::PAYMENT_METHOD,
			'text' => JText::_(self::OPTION),
			'tax_text' => $tax_text,
		);
    }
        
    /**
     * Show payment form.
     *
     * @access public
     * 
     * @param object $vars
     */
    public function onRsdirectoryshowForm($vars)
    {
		// Do a few checks and exit the function if the conditions are not met.
		if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
			return;
			
			
		// Get PayPal details.
		$paypal_email = $this->params->get('paypal_email', '');
		$paypal_return = $this->params->get('return_url', '');
		$paypal_lang = $this->params->get('paypal_lang', 'US');
			
			
		$lang = JFactory::getLanguage();
		$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
		$lang->load('com_rsdirectory', JPATH_SITE);
			
		// Validate the return url.
		if ( substr($paypal_return, 0, 4) != 'http' )
		{
			$paypal_return = '';
		}
			
			
		// Set the PayPal url.
		$paypal_url = $this->params->get('paypal_mode') ? 'https://www.paypal.com/cgi-bin/webscr' : 'https://www.sandbox.paypal.com/cgi-bin/webscr';
			
		self::outputTransactionDetails($vars);
			
		?>
			
		<p style="margin: 10px;font-weight: bold;"><?php echo JText::_('PLG_SYSTEM_RSDIRECTORYPAYPAL_REDIRECTING'); ?></p>
			
		<form id="paypalForm" method="post" action="<?php echo $paypal_url; ?>">
			<input type="hidden" name="business" value="<?php echo strtolower( self::escape($paypal_email) ); ?>" />
			<input type="hidden" name="item_name" value="<?php echo self::escape($vars->item_name); ?>" />
			<input type="hidden" name="currency_code" value="<?php echo self::escape($vars->currency); ?>" />
			<input type="hidden" name="charset" value="utf-8" />
			<input type="hidden" name="notify_url" value="<?php echo JRoute::_( JURI::root() . 'index.php?rsdirectorypaypal=1'); ?>" />
			<input type="hidden" name="custom" value="<?php echo self::escape($vars->hash); ?>" />
			<?php echo $paypal_return ? '<input type="hidden" name="return" value="' . self::escape($paypal_return) . '" />' : ''; ?>
			<input type="hidden" name="lc" value="<?php echo self::escape($paypal_lang); ?>" />
			<input type="hidden" name="no_shipping" value="1" />
			<input type="hidden" name="cmd" value="_xclick" />
			<input type="hidden" name="amount" value="<?php echo self::formatPrice($vars->price); ?>" />
			<input type="hidden" name="quantity" value="1" />
			<input type="hidden" name="bn" value="RSJoomla_SP" />
			<?php
				
			if ($vars->tax_value)
			{
				if ($vars->tax_type == 'percent')
				{
					echo '<input type="hidden" name="tax_rate" value="' . $vars->tax_value . '" />';
				}
				else
				{
					echo '<input type="hidden" name="tax" value="' . $vars->tax_value . '" />';
				}
			}
				
			?>
		</form>
			
		<script type="text/javascript">
			function paypalFormSubmit()
			{
				document.getElementById('paypalForm').submit();
			}
				
			try
			{
				window.addEventListener ? window.addEventListener('load', paypalFormSubmit, false) : window.attachEvent('onload', paypalFormSubmit);
			}
			catch (err)
			{
				paypalFormSubmit();
			}
		</script>
			
		<?php
    }
         
    /**
     * Build post data.
     *
     * @access private
     * 
     * @static
     * 
     * @return string
     */
    private static function _buildPostData()
    {
		// Read the post from PayPal system and add 'cmd'.
		$request = 'cmd=_notify-validate';
			
		// Reading raw POST data from input stream. reading post data from $_POST may cause serialization issues since POST data may contain arrays.
		$raw_post_data = file_get_contents('php://input');
			
		if ($raw_post_data)
		{
			$raw_post_array = explode('&', $raw_post_data);
			$myPost = array();
				
			foreach ($raw_post_array as $keyval)
			{
				$keyval = explode ('=', $keyval);
					
				if ( count($keyval) == 2 )
				{
					$myPost[$keyval[0]] = urldecode($keyval[1]);
				}
			}

			$get_magic_quotes_gpc = function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc();
				
			foreach ($myPost as $key => $value)
			{
				if ($key == 'limit' || $key == 'limitstart' || $key == 'option')
					continue;
					
				$value = $get_magic_quotes_gpc ? urlencode( stripslashes($value) ) : urlencode($value);
					
				$request .= "&$key=$value";
			}
		}
		else
		{
			foreach ($_POST as $key => $value)
			{
				if ($key == 'limit' || $key == 'limitstart' || $key == 'option')
					continue;
					
				$value = urlencode($value);
				$request .= "&$key=$value";
			}
		}
			
		return $request;
    }
		
    /**
     * Process the form.
     *
     * @access private
     */
    private function processForm()
    {
		// Get the hash.
		$hash = isset($_POST['custom']) ? $_POST['custom'] : '';
			
		// Do a few checks and exit the function if the conditions are not met.
		if ( !$this->canRun() || !$hash )
			return;
			
			
		$lang = JFactory::getLanguage();
		$lang->load(self::EXTENSION, JPATH_ADMINISTRATOR);
			
			
		// Load the RSDirectory! helper.
		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/rsdirectory.php';
			
		// Exit the function if no transaction was found or if the transaction was already finalized.
		$user_transaction = RSDirectoryHelper::getUserTransaction($hash);
			
		// Exit the function if no transaction was found or if the transaction was already finalized.
		if (!$user_transaction || $user_transaction->status == 'finalized')
			return;
			
		$item_name = $_POST['item_name'];
		$item_number = $_POST['item_number'];
		$payment_status = $_POST['payment_status'];
		$payment_amount = self::formatPrice($_POST['mc_gross']);
		$payment_currency = $_POST['mc_currency'];
		$txn_id = $_POST['txn_id'];
		$txn_type = strtoupper($_POST['txn_type']);
		$receiver_email = $_POST['receiver_email'];
		$payer_email = $_POST['payer_email'];
		$business = $_POST['business'];

		$paypal_email = $this->params->get('paypal_email', '');
			
		// Post back to PayPal system to validate.
		$url = $this->params->get('paypal_mode') ? 'https://www.paypal.com/cgi-bin/webscr' : 'https://www.sandbox.paypal.com/cgi-bin/webscr';
		$request_query = self::_buildPostData();

		// pass the request as array values
		parse_str($request_query, $request_array);

		$http = JHttpFactory::getHttp();

		// hold the connection response
		$response = '';

		try
		{
			$request = $http->post($url, $request_array, array('Host: www.paypal.com'), 5);

			$code = $request->code;

			if ($code != 200)
			{
				throw new Exception(sprintf('Connection Error: %d', $request->code));
			}

			// set the response
			$response = (string) $request->body;

			if (!strlen($response))
			{
				throw new Exception('Response does not contain a valid body');
			}
		}
		catch (Exception $e)
		{
			$errstr = $e->getMessage();
		}
			
		// Initialize the transaction log.
		$log = array();
			
		$log[] = 'Receiving a new transaction from Paypal.';
			
		if ( $this->params->get('paypal_mode', 0) == 0 )
		{
			$log[] = 'Demo mode is on.';
		}

		$email_custom_fields = array(
			'price-credits' 		=> $user_transaction->price,
			'tax' 					=> $user_transaction->tax,
			'tax-type' 				=> $user_transaction->tax_type,
			'tax-value' 			=> $user_transaction->tax_value,
			'total' 				=> $user_transaction->total,
			'credit-title' 			=> $user_transaction->credit_title,
			'currency' 				=> $user_transaction->currency,
			'credits' 				=> $user_transaction->credits,
			'payment-method' 		=> $user_transaction->gateway,
			'date-purchased' 		=> $user_transaction->date_created,
		);

		$send_failed_email = true;

		require_once JPATH_ADMINISTRATOR . '/components/com_rsdirectory/helpers/email.php';
			
		if (strlen($response))
		{
			if ( strcmp($response, 'VERIFIED') == 0 )
			{
				$log[] = 'PayPal reported a valid transaction.';
				$log[] = 'Payment status is ' . ( !empty($payment_status) ? $payment_status : 'empty' ) . '.';

				if ($business == $paypal_email) {
					switch ($txn_type) {
						case 'SUBSCR_PAYMENT':
						case 'CART':
						case 'EXPRESS_CHECKOUT':
						case 'VIRTUAL_TERMINAL':
						case 'WEB_ACCEPT':
						case 'SEND_MONEY':

							// Format the credits package price.
							$price = self::formatPrice($user_transaction->total);

							// Proceed if the currency and amount are ok.
							if ($user_transaction->currency == $payment_currency && $price <= $payment_amount) {
								// Set the transaction params.
								$params = array();

								if (!empty($_POST)) {
									foreach ($_POST as $key => $value) {
										$params[] = "$key=$value";
									}
								}

								$new_user_transaction = (object)array(
										'id' => $user_transaction->id,
										'gateway_order_number' => $txn_id,
										'gateway_order_type' => $txn_type,
										'gateway_params' => $params ? implode("\n", $params) : '',
										'date_finalized' => JFactory::getDate()->toSql(),
										'finalized' => 1,
										'status' => 'finalized',
								);

								$db = JFactory::getDbo();

								$db->updateObject('#__rsdirectory_users_transactions', $new_user_transaction, 'id');

								RSDirectoryCredits::addUserCredits($user_transaction->user_id, $user_transaction->credits);

								// set the email custom fields
								$email_custom_fields['gateway-order-number'] = $new_user_transaction->gateway_order_number;
								$email_custom_fields['gateway-order-type'] 	 = $new_user_transaction->gateway_order_type;
								$email_custom_fields['date-finalized'] 		 = $new_user_transaction->date_finalized;
								$email_custom_fields['status'] 				 = JText::_('COM_RSDIRECTORY_TRANSACTION_STATUS_'.strtoupper($new_user_transaction->status));


								// Get entry.
								if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
									// Get the number of unpaid entry credits.
									$cost = RSDirectoryCredits::getUnpaidEntryCreditsSum($entry->id);

									// Get the total number of credits available to the user
									$user_credits = RSDirectoryCredits::getUserCredits($user_transaction->user_id);

									// Get plugin entry publishing param.
									$publish_entry = (int) $this->params->get('publish_entry', 0);

									// Verify if this entry can be published automatically.
									$automatically_publish_entry = $publish_entry == 1 || ($publish_entry == 2 && RSDirectoryHelper::checkUserPermission('auto_publish_entries', $user_transaction->user_id));

									// Check if enough credits are available.
									$has_enough_credits = $user_credits == 'unlimited' || $cost <= $user_credits;

									if ($automatically_publish_entry && $has_enough_credits) {
										// Holds emails that will be sent later on.
										$emails = array('credit_purchase_processed_finalized_with_entry');

										// Setup new entry data that should be overridden.
										$entry_data = (object)array(
												'id' => $entry->id,
												'paid' => 1,
												'published' => 1,
												'published_time' => JFactory::getDate()->toSql()
										);

										// Calculate expiry time.
										if ($entry->period) {
											$expiry_time = JFactory::getDate($entry_data->published_time);
											$expiry_time->modify("+{$entry->period} days");

											$entry_data->expiry_time = $expiry_time->toSql();
										} else {
											$entry_data->expiry_time = $db->getNullDate();
										}

										// Flag to send emails if the entry wasn't published and now it is.
										if (!$entry->published) {
											$emails[] = 'publish_entry';
										}

										// Subtract number of credits if the user doesn't have unlimited credits
										if ($user_credits != 'unlimited') {
											// Charge user.
											$query = $db->getQuery(true)
													->update($db->qn('#__rsdirectory_users'))
													->set($db->qn('credits') . ' = ' . $db->qn('credits') . ' - ' . $db->q($cost))
													->where($db->qn('user_id') . ' = ' . $db->q($user_transaction->user_id));

											$db->setQuery($query);
											$db->execute();
										}

										// Mark entry credits as being paid.
										$query = $db->getQuery(true)
												->update($db->qn('#__rsdirectory_entries_credits'))
												->set($db->qn('paid') . ' = ' . $db->q($entry_data->paid))
												->where($db->qn('entry_id') . ' = ' . $db->q($entry_data->id));

										$db->setQuery($query);
										$db->execute();

										// Save entry data.
										$db->updateObject('#__rsdirectory_entries', $entry_data, 'id');

										$log[] = 'Automatically published entry #'.$entry->id.'.';

										// Send emails.
										if ($emails) {
											// Get the form associated to the category.
											$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

											// Get the published form fields assigned to the form.
											$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

											// Send emails
											foreach ($emails as $email) {
												RSDirectoryEmail::getInstance($email, $form_fields, $entry, $form, ($email == 'credit_purchase_processed_finalized_with_entry' ? $email_custom_fields : null))->send();
												$log[] = "Sent $email.";
											}
										}
									} else {
										// Get the form associated to the category.
										$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

										// Get the published form fields assigned to the form.
										$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

										RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
										$log[] = "Sent credit_purchase_processed_finalized_with_entry.";
									}
									$send_failed_email = false;
								} else {
									$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
									RSDirectoryEmail::getInstance('credit_purchase_processed_finalized_no_entry', null, $entry, null, $email_custom_fields)->send();
									$log[] = "Sent credit_purchase_processed_finalized_no_entry.";
									$send_failed_email = false;
								}

								$log[] = 'Successfully added the payment to the database.';
							} else {
								$log[] = "Expected an amount of $price $user_transaction->currency. PayPal reports this payment is $payment_amount $payment_currency. Stopping.";
							}

							break;
					}
				} else {
					$log[] = 'Expected payment to '.$paypal_email.', received payment to '.$business.'. Stopping.';
				}
			}
			else if ( strcmp($response, 'INVALID') == 0 )
			{
				// Log for manual investigation.
				$log[] = "Could not verify transaction authenticity. PayPal said it's invalid.";
				$log[] = "String sent to PayPal is $request_query";
			}
		}
		else
		{
			$log[] = "Could not connect to $url in order to verify this transaction. Error reported is: $errstr";
		}

		// send the refusal email if is set
		if ($send_failed_email) {
			if ($user_transaction->entry_id && ($entry = RSDirectoryHelper::getEntry($user_transaction->entry_id))) {
				// Get the form associated to the category.
				$form = RSDirectoryHelper::getCategoryInheritedForm($entry->category_id);

				// Get the published form fields assigned to the form.
				$form_fields = RSDirectoryHelper::getFormFields($form->id, 1, 1);

				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_with_entry', $form_fields, $entry, $form, $email_custom_fields)->send();
			} else {
				$entry = (object) array('skip_entry'=>true, 'user_id' => $user_transaction->user_id);
				RSDirectoryEmail::getInstance('credit_purchase_processed_refused_no_entry', null, $entry, null, $email_custom_fields)->send();
			}
		}
			
		RSDirectoryHelper::saveTransactionLog($user_transaction->id, $log);
    }
		
	/**
     * Get taxa data.
     *
     * @access public
     *
     * @param object $vars
     *
     * @return array
     */
    public function onRsdirectorygetTaxData($vars)
    {
        if ( !$this->canRun() || !isset($vars->method) || $vars->method != self::PAYMENT_METHOD )
            return;
            
        // Calculate the tax.
        $tax_type = $this->params->get('tax_type');
		$tax_value = $this->params->get('tax_value');
        $tax = $tax_value && $tax_type ? $vars->price * ($tax_value/100) : $tax_value;
            
        $data = array(
            'tax_type' => $tax_type ? 'percent' : 'fixed',
            'tax_value' => $tax_value,
            'tax' => $tax,
        );
			
		return $data;
    }
}